  unit IWCompEdit;

{PUBDIST}

interface

uses
  {$IFDEF VSNET}
    System.ComponentModel, System.Drawing, IWNetComponent,
  {$ELSE}
  {$IFDEF Linux}
  IWCLXComponent,
  {$ELSE}
  IWVCLComponent,
  {$ENDIF}
  {$IFDEF Linux}QForms, {$ELSE}Forms, {$ENDIF}
  {$ENDIF}
  {$IFDEF CLR}
    IWNetFileParser,
  {$ELSE}
    IWFileParser,
  {$ENDIF}
  {$IFDEF Linux}QGraphics, {$ELSE}Graphics, {$ENDIF}
  Classes,
  IWControl,
  IWBaseControl, IWTypes, IWScriptEvents, IWHTMLTag, IWColor,
  IWRenderContext, IWBaseInterfaces, IWCompButton;

type
  TIWCustomEdit = class(TIWCustomControl, IIWInputControl, IIWSubmitControl)
  protected
    FSubmitParam : String;  
    FAlignment: TAlignment;
    FBGColor: TIWColor;
    FFocusColor: TIWColor;
    FMaxLength: Integer;
    FOnSubmit: TNotifyEvent;
    FPasswordPrompt: Boolean;
    FRequired: Boolean;
    FReadOnly: Boolean;
    FTagType: string;
    //
    procedure DoSubmit;
    procedure HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents); override;
    procedure SetValue(const AValue: string); virtual;
    procedure Submit(const AValue: string); override;
    procedure SetPasswordPrompt(const AValue: Boolean);
    procedure SetBGColor(const Value: TIWColor);
    procedure SetAlignment(const Value: TAlignment);

    procedure InitControl; override;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    function RenderStyle(AContext: TIWBaseHTMLComponentContext): string; override;
    property PasswordPrompt: Boolean read FPasswordPrompt write SetPasswordPrompt;
    function GetSubmitParam : String;    
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;

    property BGColor: TIWColor read FBGColor write SetBGColor;
    property FocusColor: TIWColor read FFocusColor write FFocusColor;
    property DoSubmitValidation;
    property Editable;

    {$IFDEF CLR}
    property WebFont;
    {$ELSE}
    property Font;
    {$ENDIF}
    property ExtraTagParams;
    property FriendlyName;

    property MaxLength: Integer read FMaxLength write FMaxLength;

    property ReadOnly: Boolean read FReadOnly write FReadOnly;

    property Required: Boolean read FRequired write FRequired;
    property ScriptEvents;
    property TabOrder;

    property OnSubmit: TNotifyEvent read FOnSubmit write FOnSubmit;
  end;

{$IFDEF VSNET}
{$R icons\Atozed.Intraweb.IWCompEdit.TIWEdit.bmp}
  TIWEdit = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWEdit), 'TIWEdit.bmp')]
{$ENDIF}
  TIWEdit = class(TIWCustomEdit)
  public
    procedure Clear;
  published
    property PasswordPrompt;

    property Text;
  end;

  TReceivedFileEvent = procedure(ASender: TObject; const AFilename: string) of object;

  TIWFileButtonPosition = (ubpBelowEdit, ubpNextToBrowse);

  TIWFileUploadButton = class(TPersistent)
  protected
    FColor: TIWColor;
    FCaption: string;
    FPosition: TIWFileButtonPosition;
    FVisible: boolean;
    FOnChange: TNotifyEvent;
    procedure DoChange;
    procedure SetPosition(Value: TIWFileButtonPosition);
    procedure SetColor(Value: TIWColor);
    procedure SetCaption(Value: string);
    procedure SetVisible(Value: boolean);
  public
    procedure Assign(ASource: TPersistent); override;

    constructor Create;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Color: TIWColor read FColor write SetColor;
    property Caption: string read FCaption write SetCaption;
    property Position: TIWFileButtonPosition read FPosition write SetPosition;
    property Visible: boolean read FVisible write SetVisible;
  end;

  TIWCustomFile = class(TIWCustomControl, IIWInputControl, IIWSubmitControl)
  protected
    FSubmitParam : String;
    FBGColor: TIWColor;
    FFrameBGColor: TIWColor;
    FUploadButton: TIWFileUploadButton;
    FContentType: string;
    FFileName: string;
    FOnReceivedFile: TReceivedFileEvent;
    FButtonHeight: Integer;

    FFile: THTTPFile;

    FSubmitControl:TIWButton;

    function GetContentType: string;
    procedure DoReceivedFile(const AFileName: string);
    function GetSize: Integer;
    procedure SetValue(const AValue: string); virtual;
    procedure SetFileValue(AFile: THTTPFile); override;

    procedure InitControl; override;
    procedure Submit(const AValue: string); override;
    procedure DoUploadButtonChange(Sender: TObject);
    procedure SetBGColor(const Value: TIWColor);
    procedure SetFrameBGColor(const Value: TIWColor);
    procedure Notification(AComponent: TPlatformComponent; AOperation: TOperation); override;
{$IFDEF CLR}
    strict protected
{$ELSE}
  protected
{$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    function RenderStyle(AContext: TIWBaseHTMLComponentContext): string; override;
    function GetSubmitParam : String;
    procedure SaveToFile(AFilename: string = '');

    procedure SaveToStream(AStream: TStream);

    property ContentType: string read GetContentType;

    property Filename: string read FFileName;
    property ButtonHeight: Integer read FButtonHeight write FButtonHeight;
    property BGColor: TIWColor read FBGColor write SetBGColor;
    property FrameBGColor: TIWColor read FFrameBGColor write SetFrameBGColor;
    property SubmitControl:TIWButton read FSubmitControl write FSubmitControl;
  published
    property TabOrder;
    property OnReceivedFile: TReceivedFileEvent read FOnReceivedFile write FOnReceivedFile;
  end;

  TIWFile = class(TIWCustomFile)
  published
    property UploadButton: TIWFileUploadButton read FUploadButton write FUploadButton;
    property FriendlyName;
    {$IFDEF CLR}
    property WebFont;
    {$ELSE}
    property Font;
    {$ENDIF}
    property BGColor;
    property ButtonHeight;
    property FrameBGColor;
    property SubmitControl;
  end;

implementation

uses
  SysUtils, 
  IWAppForm, IWApplication, IWCompLabel, IWResourceStrings, IWServerControllerBase,
  SWSystem, SWStrings, IWGlobal, IWForm, IWRegion;

procedure TIWCustomEdit.SetValue(const AValue: string);
begin
  Text := AValue;
  Invalidate;
end;

function TIWCustomEdit.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

function TIWCustomEdit.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  LLabel: TIWLabel;
begin
  result := nil;
  if Editable then begin
    Result := TIWHTMLTag.CreateTag('INPUT'); try
      Result.AddStringParam('TYPE', FTagType);
      Result.AddStringParam('NAME', HTMLName);
      if MaxLength > 0 then begin
        Result.AddIntegerParam('MAXLENGTH', MaxLength);
      end;
      Result.AddStringParam('VALUE', TextToHTML(Text));
      Result.Add(iif(ReadOnly or IsDesignMode, 'READONLY'));
      if Required then begin
        (AContext as TIWComponent40Context).AddValidation(HTMLName + 'IWCL != null && ' + HTMLName + 'IWCL.value.length==0'
          , Format(RSValidationRequeredField, [FriendlyName]));
      end;
    except FreeAndNil(Result); raise; end;
  end else begin
    LLabel := TIWLabel.Create{$IFNDEF VSNET}(Self){$ENDIF}; try
      LLabel.Name := Name;
      if FPasswordPrompt then begin
        LLabel.Caption := StringOfChar('*', Length(Text));
      end else begin
        LLabel.Caption := Text;
      end;
      LLabel.WebFont.Assign(WebFont);
      LLabel.Width := Width;
      LLabel.Height := Height;
      Result := LLabel.RenderHTML(AContext);
    finally FreeAndNil(LLabel); end;
  end;
end;

procedure TIWCustomEdit.DoSubmit;
begin
  if Assigned(OnSubmit) then begin
    OnSubmit(Self);
  end;
end;

procedure TIWCustomEdit.Submit(const AValue: string);
begin
  FSubmitParam := AValue;
  DoSubmit;
end;

function TIWCustomEdit.RenderStyle(AContext: TIWBaseHTMLComponentContext): string;
begin
  Result := inherited RenderStyle(AContext)
    + iif(toTColor(WebFont.Color) <> clNone, 'color:' + ColorToRGBString(WebFont.Color) + ';');
  if toTColor(BGColor) <> clNone then begin
    Result := Result + 'background-color:' + ColorToRGBString(BGColor) + ';';
  end;
  case Alignment of
    taLeftJustify: Result := Result + 'text-align:left;';
    taCenter: Result := Result + 'text-align:center;';
    taRightJustify: Result := Result + 'text-align:right;';
  end;
end;

procedure TIWCustomEdit.HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents);
begin
  inherited HookEvents(AContext, AScriptEvents);
  if Assigned(OnSubmit) then begin
    AScriptEvents.HookEvent('OnKeyPress', 'var res = CheckReturnKey('
      + iif(AContext.WebApplication.Browser = brIE, 'event.keyCode', 'event.which')
      + ', ''' + HTMLName + ''', ' + iif(DoSubmitValidation, 'true', 'false') + ')' + EOL
      // + iif(AContext.WebApplication.Browser <> brIE, 'if (!res) { noSecondSubmit = false; }'+ EOL)
      + 'return res;');
  end;
  if toTColor(FocusColor) <> clNone then begin
    AScriptEvents.HookEvent('OnFocus', 'this.style.backgroundColor=''' + ColorToRGBString(FocusColor) + ''';');
    AScriptEvents.HookEvent('OnBlur', 'this.style.backgroundColor=''' + ColorToRGBString(BGColor) + ''';');
  end;
end;

procedure TIWCustomEdit.InitControl;
begin
  inherited;
  FFocusColor := fromTColor(clNone);
  FBGColor := fromTColor(clNone);
  FNeedsFormTag := True;
  FTagType := 'TEXT';
  Height := 21;
  Width := 121;
  FAlignment := taLeftJustify;
  set_RenderSize(true);
  FCanReceiveFocus := True;
end;

{ TIWCustomFile }

function TIWCustomFile.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

procedure TIWCustomFile.DoReceivedFile(const AFileName: string);
begin
  if Assigned(FOnReceivedFile) then begin
    OnReceivedFile(Self, AFileName);
  end;
end;

function TIWCustomFile.GetSize: Integer;
begin
  result := 0;
  if Assigned(FFile) then
    Result := FFile.Stream.Size;
end;

function TIWCustomFile.GetContentType: string;
begin
  Result := '';
  if Assigned(FFile) then
    Result := FFile.ContentType;
end;

procedure TIWCustomFile.Notification(AComponent: TPlatformComponent; AOperation: TOperation);
begin
  if (AOperation = opRemove) and (AComponent = FSubmitControl) then begin
    FSubmitControl := nil;
  end else begin
    inherited Notification(AComponent, AOperation);
  end;
end;

procedure TIWCustomFile.SaveToFile(AFilename: string = '');
begin
  if AFileName = '' then begin
    AFileName := Filename;
  end;
  if Assigned(FFile) then begin
    FFile.SaveToFile(AFileName);
  end;
end;

procedure TIWCustomFile.SaveToStream(AStream: TStream);
begin
  if Assigned(FFile) then begin
    FFile.SaveToStream(AStream);
  end;
end;

procedure TIWCustomEdit.SetPasswordPrompt(const AValue: Boolean);
begin
  FTagType := IIF(AValue, 'PASSWORD', 'TEXT');
  FPasswordPrompt := AValue;
end;

procedure TIWCustomEdit.SetBGColor(const Value: TIWColor);
begin
  FBGColor := Value;
  Invalidate;
end;

procedure TIWCustomEdit.SetAlignment(const Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TIWCustomFile.SetValue(const AValue: string);
begin
  FFilename := AValue;
end;

function TIWCustomFile.RenderHTML(
  AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  LFrameBody: TIWHTMLTag;
  LUploadForm: string;
  LAction: string;
  LBGColor: IWColor.TIWColor;
begin
  Result := TIWHTMLTag.CreateTag('IFRAME');
  try
    LUploadForm := GServerController.NewCacheFile('html');

    with Result do
    begin
      AddStringParam('SRC', AContext.WebApplication.InternalURLBase + '/cache/' + ExtractFilename(LUploadForm));
      AddStringParam('frameborder', '0');
    end;

    LFrameBody := TIWHTMLTag.CreateTag('body');
    try
      with LFrameBody do
      begin
        AddStringParam('TOPMARGIN', '0');
        AddStringParam('LEFTMARGIN', '0');
        if FFrameBGColor <> fromTColor(clNone) then begin
          AddStringParam('BGCOLOR', ColorToRGBString(FFrameBGColor));
        end else begin
          LBGColor := fromTColor(clNone);
          if Parent is TIWForm then begin
            LBGColor := (Parent as TIWForm).BGColor;
          end else if Parent is TIWRegion then begin
            LBGColor := (Parent as TIWRegion).WebColor;
          end;
          if LBGColor <> fromTColor(clNone) then begin
            if (AContext.WebApplication.Browser = brIE) and
               (FFrameBGColor = fromTColor(clNone)) and
               (LBGColor <> fromTColor(clNone)) then begin
              AddStringParam('BGCOLOR', ColorToRGBString(LBGColor));
            end;
          end;
        end;
      end;
      with LFrameBody.Contents.AddTag('form') do begin
        AddStringParam('method', 'POST');
        AddStringParam('enctype', 'multipart/form-data');
        AddStringParam('name', 'HiddenSubmitForm');
        if not IsDesignMode then begin
          AddStringParam('onsubmit', 'top.GSubmitting = true; top.SetBusy(true);');
        end;
        with Contents.AddTag('input') do
        begin
          AddStringParam('type', 'file');
          AddStringParam('name', HTMLName);
          AddStringParam('style', WebFont.FontToStringStyle(AContext.WebApplication.Browser) + 'height:' + IntToStr(FButtonHeight) + ';');
          if FBGColor <> fromTColor(clNone) then
          begin
            Params.Values['style'] := Params.Values['style'] + 'background-color:' + ColorToRGBString(FBGColor);
          end;
          if IsDesignMode or not Enabled then begin
            Add('DISABLED');
          end;
        end;
        with Contents.AddTag('INPUT') do
        begin
          AddStringParam('TYPE', 'HIDDEN');
          AddStringParam('NAME', 'IW_ActionParam');
          AddStringParam('value', HTMLName);
        end;

        if FUploadButton.Visible then begin
          if FUploadButton.Position = ubpBelowEdit then
          begin
            Contents.AddTag('BR');
          end;

          with Contents.AddTag('INPUT') do
          begin
            AddStringParam('TYPE', 'submit');
            AddStringParam('NAME', 'Upload');
            AddStringParam('value', FUploadButton.Caption);
            AddStringParam('style', WebFont.FontToStringStyle(AContext.WebApplication.Browser) + 'height:' + IntToStr(FButtonHeight) + ';');
            if FUploadButton.Color <> fromTColor(clNone) then
            begin
              Params.Values['style'] := Params.Values['style'] + 'background-color:' + ColorToRGBString(FUploadButton.Color);
            end;
            if IsDesignMode or not Enabled then begin
              Add('DISABLED');
            end;
          end;
        end;

        if not IsDesignMode then begin
          with AContext.WebApplication do
            case GServerController.SessionTrackingMethod of
              tmCookie: begin
                  LAction := AppURLBase + '/upload/';
                  with Contents.AddTag('INPUT') do
                  begin
                    AddStringParam('TYPE', 'HIDDEN');
                    AddStringParam('NAME', 'IW_TrackID_');
                    AddIntegerParam('VALUE', TrackID);
                  end;
                end;
              tmURL: begin
                  LAction := AppURLBase + '/upload/' + IntToStr(TrackID) + '/' + AppID + SessionTimeStamp;
                  Invalidate;
                end;
              tmHidden: begin
                  LAction := AContext.WebApplication.AppURLBase + '/upload';
                  with Contents.AddTag('INPUT') do
                  begin
                    AddStringParam('TYPE', 'HIDDEN');
                    AddStringParam('NAME', 'IW_SessionID_');
                    AddStringParam('VALUE', AppID + SessionTimeStamp);
                  end;
                  with Contents.AddTag('INPUT') do
                  begin
                    AddStringParam('TYPE', 'HIDDEN');
                    AddStringParam('NAME', 'IW_TrackID_');
                    AddIntegerParam('VALUE', TrackID);
                  end;
                  Invalidate;
                end;
            end;
          AddStringParam('action', LAction);
        end;
      end;
      FileMake(LUploadForm, '<HTML><HEAD></HEAD>' + LFrameBody.Render + '</HTML>');
    finally
      LFrameBody.Free;
    end;
  except
    FreeAndNil(Result);
  end;
end;

procedure TIWCustomFile.Submit(const AValue: string);
begin
  FSubmitParam := AValue;
  // Does nothing just to protect from exception in ExecuteForm
end;

{ TIWEdit }

procedure TIWEdit.Clear;
begin
  Text := '';
end;

procedure TIWCustomFile.InitControl;
begin
  inherited;
  Width := 260;
  Height := 80;
  FFile := nil;
  set_RenderSize(true);
  ControlEncode := ceMPFormData;
  FBGColor := fromTColor(clNone);
  FFrameBGColor := fromTColor(clNone);
  FUploadButton := TIWFileUploadButton.Create;
  FUploadButton.OnChange := DoUploadButtonChange;
  FButtonHeight := 21;
end;

procedure TIWCustomFile.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FUploadButton);
  inherited;
end;

function TIWCustomFile.RenderStyle(
  AContext: TIWBaseHTMLComponentContext): string;
begin
  result := inherited RenderStyle(AContext);
  result := result + 'border-width:0;border-style:none;overflow:hidden;';
  if toTColor(FBGColor) <> clNone then begin
    Result := Result + 'background-color: ' + ColorToRGBString(FBGColor) + ';';
  end;
end;

procedure TIWCustomFile.SetFileValue(AFile: THTTPFile);
begin
  FFile := AFile;
  DoReceivedFile(FFileName);
end;

procedure TIWCustomFile.DoUploadButtonChange(Sender: TObject);
begin
  Invalidate;
end;

procedure TIWCustomFile.SetFrameBGColor(const Value: TIWColor);
begin
  FFrameBGColor := Value;
  Invalidate;
end;

procedure TIWCustomFile.SetBGColor(const Value: TIWColor);
begin
  if FBGColor <> Value then
  begin
    FBGColor := Value;
    Invalidate;
  end;
end;

{ TIWFileUploadButton }

procedure TIWFileUploadButton.Assign(ASource: TPersistent);
begin
  if ASource is TIWFileUploadButton then begin
    with ASource as TIWFileUploadButton do begin
      FCaption := Self.Caption;
      FColor := Self.Color;
    end;
  end else begin
    inherited Assign(ASource);
  end;
end;

constructor TIWFileUploadButton.Create;
begin
  inherited;
  Caption := 'Upload';
  FColor := fromTColor(clNone);
  FVisible := true;
  Position := ubpNextToBrowse;
end;

procedure TIWFileUploadButton.DoChange;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TIWFileUploadButton.SetPosition(Value: TIWFileButtonPosition);
begin
  if FPosition <> Value then
  begin
    FPosition := Value;
    DoChange;
  end;
end;

procedure TIWFileUploadButton.SetColor(Value: TIWColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    DoChange;
  end;
end;

procedure TIWFileUploadButton.SetCaption(Value: string);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    DoChange;
  end;
end;

procedure TIWFileUploadButton.SetVisible(Value: boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    DoChange;
  end;
end;

end.
