unit IWCompLabel;
{PUBDIST}

interface

uses
  {$IFDEF VSNET}
  System.ComponentModel, System.Drawing,
  {$ELSE}
  {$IFDEF Linux}QControls, {$ELSE}Controls, {$ENDIF}
  {$ENDIF}
  {$IFDEF Linux}QGraphics, {$ELSE}Graphics, {$ENDIF}
  Classes,
  IWControl, IWHTMLTag, IWColor, IWRenderContext;

type
  TIWCustomLabel = class(TIWCustomControl)
  protected
    FAlignment: TAlignment;
    FRawText: Boolean;
    FNoWrap: Boolean;
    //
    procedure SetAutoSize(Value: Boolean); override;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetRawText(Value: Boolean);
    procedure InitControl; override;
    procedure SetBGColor(Value: TIWColor);
    function GetBGColor : TIWColor;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;

    property AutoSize;
    property RawText: Boolean read FRawText write SetRawText;
    {$IFNDEF CLR}
    property WebFont;
    {$ENDIF}
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property BGColor : TIWColor read GetBGColor write SetBGColor;
    {$IFDEF CLR}
    property WebFont;
    {$ELSE}
    property Font;
    {$ENDIF}
    property NoWrap: Boolean read FNoWrap write FNoWrap;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWCompLabel.TIWLabel.bmp}
  TIWLabel = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWLabel), 'TIWLabel.bmp')]
  {$ENDIF}
  TIWLabel = class(TIWCustomLabel)
  published
    property AutoSize;
    property FriendlyName;

    property Caption;
    property RawText;
  end;

implementation

uses
  SysUtils;

function TIWCustomLabel.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  LStyle: string;
begin
  LStyle := 'style=';
  Result := TIWHTMLTag.CreateTag('SPAN');
  try
    if not FRawText then
    begin
      if NoWrap then
      begin
        Result.Contents.AddText('<NOBR>' + TextToHTML(Caption, true, true) + '</NOBR>');
      end
      else
      begin
        Result.Contents.AddText(TextToHTML(Caption, true, true));
      end;
    end
    else
    begin
      if NoWrap then
      begin
        Result.Contents.AddText('<NOBR>'+ Caption + '</NOBR>');
      end
      else
      begin
        Result.Contents.AddText(Caption);
      end;
    end;
    if not AutoSize then
    begin
      case Alignment of
        taLeftJustify: LStyle := LStyle + 'text-align:left;';
        taCenter: LStyle := LStyle + 'text-align:center;';
        taRightJustify: LStyle := LStyle + 'text-align:right;';
      end;
    end;
    if toTColor(WebColor) <> clNone then
    begin
      LStyle := LStyle + 'background-color:' + ColorToRGBString(WebColor) +';';
    end;
    if LStyle <> 'style=' then
    begin
      Result.Add(LStyle);
    end;
  except
    FreeAndNil(Result);
    raise;
  end;
end;

procedure TIWCustomLabel.InitControl;
begin
  inherited;
  Height := 21;
  Width := 121;
  AutoSize := TRUE;
  FRawText := FALSE;
  FAlignment := taLeftJustify;
  WebColor := fromTColor(clNone);
end;

procedure TIWCustomLabel.SetAutoSize(Value: Boolean);
begin
  inherited SetAutoSize(Value);
  RenderSize := not Value;
end;

procedure TIWCustomLabel.SetAlignment(const Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TIWCustomLabel.SetRawText(Value: Boolean);
begin
  FRawText := Value;
end;

procedure TIWCustomLabel.SetBGColor(Value: TIWColor);
begin
  WebColor := Value;
end;

function TIWCustomLabel.GetBGColor : TIWColor;
begin
  Result := WebColor;
end;

end.

