unit IWCompListbox32;
{PUBDIST}

interface

uses
  {$IFDEF VSNET}
  System.ComponentModel, System.Drawing, IWNetClasses,
  {$ELSE}
  Classes,
      {$IFDEF Linux}
    QTypes, IWCLXComponent, IWCLXClasses,
    {$ELSE}
    IWVCLComponent, IWVCLClasses,
    {$ENDIF}
  {$IFDEF Linux}QControls,{$ELSE}Controls,{$ENDIF}
  {$ENDIF}
  {$IFDEF Linux}QGraphics,{$ELSE}Graphics,{$ENDIF}
  IWBaseControl, IWHTMLTag, IWControl32, IWScriptEvents, IWRenderContext,
  IWBaseInterfaces;

{$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWCompListbox32.TIWComboBox32.bmp}
  {$R icons\Atozed.Intraweb.IWCompListbox32.TIWListbox32.bmp}
{$ENDIF}

type
  TIWCustomListCombo32 = class(TIWCustomControl32)
  private
    FItems: TIWStringList;
  protected
    FItemIndex: Integer;
    FItemsHaveValues: Boolean;
    FRequireSelection: Boolean;
    FNoSelectionText: string;
    //
    procedure InitItems; virtual;
    procedure OnItemsChange(ASender: TObject);
    function getText: TCaption; override;
    function GetSorted: boolean;
    procedure SetItems(AValue: TIWStringList);
    function GetItems: TIWStringList;
    procedure SetItemIndex(AIndex: Integer); virtual;
    procedure SetRequireSelection(const AValue: Boolean); virtual;
    procedure SetSorted(AValue: boolean);
    procedure SetItemsHaveValues(const Value: Boolean);
    procedure InitControl; override;
    {$IFDEF VSNET}
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    {$ENDIF}
    property Items: TIWStringList read GetItems write SetItems;

    property ItemIndex: Integer read FItemIndex write SetItemIndex;

    // Do not publish sorted. DB controls use this and using it can interfere with their
    // functionality. Such as the lookuplist ones which rely on the sort order remaining the same
    // as fetched from the DB.
    //
    //The items in the TIWCustomListCombo are sorted.
    //
    //
    property Sorted: Boolean read GetSorted write SetSorted;
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  public
    procedure Clear; virtual;
    property Text: TCaption read getText write SetText;
  published
    property ExtraTagParams;
    property ItemsHaveValues: Boolean read FItemsHaveValues write SetItemsHaveValues;
    property NoSelectionText: string read FNoSelectionText write FNoSelectionText;
    property RequireSelection: Boolean read FRequireSelection write SetRequireSelection;
  end;

  TIWCustomComboBox32 = class(TIWCustomListCombo32, IIWInputControl)
  protected
    procedure InitControl; override;
    procedure SetValue(const AValue: string); virtual;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    // Needed for PaintHandlers
    property Items;
    property ItemIndex;
  published
    property Editable;
    property TabOrder;
  end;

  {$IFDEF VSNET}
  TIWComboBox32 = class;
  [ToolboxBitmap(typeof(TIWComboBox32), 'TIWComboBox32.bmp')]
  {$ENDIF}
  TIWComboBox32 = class(TIWCustomComboBox32)
  published
    property ItemIndex;
    property Items;
    property Sorted;
  end;

  TIWCustomListbox32 = class(TIWCustomListCombo32, IIWInputControl)
  protected
    FMultiSelect: Boolean;
    FSelectedList: TList;
    FTempList : TIWStringList;
    //
    procedure DoItemsChange(Sender: TObject);
    procedure DoItemsChanging(Sender: TObject);
    function GetSelected(AIndex: integer): boolean;
    procedure SetSelected(AIndex: integer; const AValue: boolean);
    procedure SetMultiSelect(const AValue: Boolean);

    procedure SetValue(const AValue: string); virtual;
    procedure InitControl; override;
    procedure InitItems; override;
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    // Needed for PaintHandlers
    property Items;
    property ItemIndex;

    property MultiSelect: boolean read FMultiSelect write SetMultiSelect;

    procedure ResetSelection;

    property Selected[AIndex: Integer]: Boolean read GetSelected write SetSelected;
  published
    property Editable;
    property TabOrder;
  end;

  {$IFDEF VSNET}
  TIWListbox32 = class;
  [ToolboxBitmap(typeof(TIWListbox32), 'TIWListbox32.bmp')]
  {$ENDIF}
  TIWListbox32 = class(TIWCustomListbox32)
  protected
    procedure SetRequireSelection(const AValue: Boolean); override;
  published
    property ItemIndex;
    property Items;
    property MultiSelect: boolean read FMultiSelect write SetMultiSelect;
    property Sorted;
  end;

implementation

uses
  SysUtils,
  {$IFNDEF Linux} Windows, {$ENDIF}
  IWCompLabel32, SWSystem, SWStrings;

{ TIWCustomListCombo32 }

procedure TIWCustomListCombo32.InitControl;
begin
  inherited;
  FNeedsFormTag := True;
  RequireSelection := True;
  FNoSelectionText := '-- No Selection --';
  FItemIndex := -1;
  Editable := True;
end;

procedure TIWCustomListCombo32.Dispose(ADispose: Boolean);
begin
  FreeandNil(FItems);
  inherited;
end;

function TIWCustomListCombo32.GetSorted: boolean;
begin
  Result := Items.Sorted;
end;

function TIWCustomListCombo32.getText: TCaption;
begin
  if ItemIndex > -1 then begin
    if ItemsHaveValues then begin
      Result := Items.Values[Items.Names[ItemIndex]];
    end else begin
      Result := Items.Strings[ItemIndex];
    end;
  end else begin
    Result := '';
  end;
end;

procedure TIWCustomListCombo32.SetItemIndex(AIndex: Integer);
begin
  {$IFDEF VSNET}
  if not Assigned(FItems) then
  {$ELSE}
  if IsLoading then
  {$ENDIF}
  begin
    // Set no matter what, it might be set (and usually is) before the items are loaded
    FItemIndex := AIndex;
  end else if AIndex < Items.Count then begin
    FItemIndex := AIndex;
    Invalidate;
  end;
end;

procedure TIWCustomListCombo32.Clear;
begin
  ItemIndex := -1;
  Items.Clear;
end;

procedure TIWCustomListCombo32.SetItems(AValue: TIWStringList);
begin
  if (AValue.Count = 0) or (AValue.Count < Items.Count) then begin
    FItemIndex := -1;
  end;
  Items.Assign(AValue);
  Invalidate;
end;

procedure TIWCustomListCombo32.SetItemsHaveValues(const Value: Boolean);
begin
  FItemsHaveValues := Value;
  Invalidate;
end;

procedure TIWCustomListCombo32.SetRequireSelection(const AValue: Boolean);
begin
  FRequireSelection := AValue;
end;

procedure TIWCustomListCombo32.SetSorted(AValue: boolean);
begin
  Items.Sorted := AValue;
  Invalidate;
end;

procedure TIWCustomListCombo32.OnItemsChange(ASender: TObject);
begin
  Invalidate;
end;

procedure TIWCustomListCombo32.InitItems;
begin
  FItems := TIWStringList.Create;
  FItems.OnChange := OnItemsChange;
end;

function TIWCustomListCombo32.GetItems: TIWStringList;
begin
  if not Assigned(FItems) then begin
    InitItems;
  end;
  result := FItems;
end;

{ TIWCustomComboBox32 }

procedure TIWCustomComboBox32.SetValue(const AValue: string);
var
  s: string;
  i: integer;
begin
  s := AValue;
  if ItemsHaveValues then begin
    for i := 0 to Items.Count - 1 do begin
      if S = Items.Values[Items.Names[i]] then begin
        ItemIndex := i;
        break;
      end;
    end;
    if i = Items.Count then begin
      ItemIndex := -1;
      Invalidate;
    end;
  end else begin
    ItemIndex := StrToIntDef(Fetch(s, ','), -1);
  end;
  Invalidate;
end;

procedure TIWCustomComboBox32.InitControl;
begin
  inherited;
  Height := 21;
  Width := 121;
end;

function TIWCustomComboBox32.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  i: Integer;
  LText: string;
  LValue: string;
begin
  Result := nil;
  if Editable then begin
    Result := TIWHTMLTag.CreateTag('SELECT'); try
      Result.AddStringParam('NAME', HTMLName);
      Result.AddIntegerParam('SIZE', 1);
      if Items.Count > 0 then begin
        if ((FItemIndex = -1) or (RequireSelection = False)) then begin
          with Result.Contents.AddTag('OPTION') do begin
            Add('SELECTED');
            AddIntegerParam('VALUE', -1);
            Contents.AddText(FNoSelectionText);
          end;
        end;
        for i := 0 to Items.Count - 1 do begin
          LText := Items.Strings[i];
          LValue := '';
          if ItemsHaveValues then begin
            LValue := LText;
            LText := Fetch(LValue, '=');
          end else begin
            LValue := IntToStr(i);
          end;
          with Result.Contents.AddTag('OPTION') do begin
            Add(iif(ItemIndex = i, 'SELECTED'));
            AddStringParam('VALUE', LValue);
            Contents.AddText(TextToHTML(LText));
          end;
        end;
      end
      else begin
        Result.Contents.AddText('');
      end;
    except FreeAndNil(Result); raise; end;
  end else begin
    with TIWLabel32.Create{$IFNDEF VSNET}(Self){$ENDIF} do try
      Name := Self.Name;
      ExtraTagParams.Assign(Self.ExtraTagParams);
      WebFont.Assign(Self.WebFont);
      Width := Self.Width;
      Height := Self.Height;
      Caption := '';
      RawText := FALSE;
      if (Self.Items.Count > 0) and (Self.ItemIndex > -1) then begin
        Caption := TextToHTML(Self.Items.Strings[Self.ItemIndex]);
      end;
      Result := RenderHTML(AContext);
    finally Free; end;
  end;
end;

{ TIWCustomListbox32 }

procedure TIWCustomListbox32.InitControl;
begin
  inherited;
  FNeedsFormTag := True;
  FSelectedList := TList.Create;
  Height := 121;
  Width := 121;

  FTempList := TIWStringList.Create;
end;

procedure TIWCustomListbox32.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FTempList);
  FreeAndNil(FSelectedList);
  inherited;
end;

procedure TIWCustomListbox32.DoItemsChange(Sender: TObject);
var
  f : integer;
  LTempSelected : TList;
begin
  if not(MultiSelect and (FSelectedList.Count > 0)) then
  begin
    Exit;
  end;

  LTempSelected := TList.Create;
  try
     for f := 0 to Pred(FTempList.Count) do
     begin
        if Items.IndexOf(FTempList.Strings[f]) >= 0 then
        begin
           if FSelectedList.IndexOf(TObject(f)) >= 0 then
           begin
              LTempSelected.Add(TObject(Items.IndexOf(FTempList.Strings[f])));
           end;
        end;
     end;
     FSelectedList.Clear;
     {$IFDEF VER130}
     FSelectedList.Capacity := LTempSelected.Count;//
     CopyMemory(FSelectedList.List, LTempSelected.List, SizeOf(LTempSelected.List^[0]) * LTempSelected.Count);
     {$ELSE}
     FSelectedList.Assign(LTempSelected);
     {$ENDIF}
  finally
     FreeAndNil(LTempSelected);
  end;
end;

procedure TIWCustomListbox32.DoItemsChanging(Sender: TObject);
begin
  if not(MultiSelect and (FSelectedList.Count > 0)) then
  begin
    Exit;
  end;

  FTempList.Assign(Items);
end;

function TIWCustomListbox32.GetSelected(AIndex: integer): boolean;
begin
  if FMultiSelect then begin
    Result := FSelectedList.IndexOf(TObject(AIndex)) > -1;
  end else begin
    Result := AIndex = ItemIndex;
  end;
end;

function TIWCustomListbox32.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  i: Integer;
  LText: string;
  LValue: string;
begin
  Result := nil;
  if Editable then begin
    Result := TIWHTMLTag.CreateTag('SELECT'); try
      Result.AddStringParam('NAME', HTMLName);
      Result.AddIntegerParam('SIZE', Height div 16);
      Result.Add(iif(FMultiSelect, 'MULTIPLE'));
      if Items.Count > 0 then begin
        if ((FItemIndex = -1) or (RequireSelection = False))
         and (FMultiSelect = False) then begin
          with Result.Contents.AddTag('OPTION') do begin
            Add('SELECTED');
            AddIntegerParam('VALUE', -1);
            Contents.AddText(FNoSelectionText);
          end;
        end;
        for i := 0 to Items.Count - 1 do begin
          LText := Items.Strings[i];
          LValue := '';
          if ItemsHaveValues then begin
            LValue := LText;
            LText := Fetch(LValue, '=');
          end else begin
            LValue := IntToStr(i);
          end;
          with Result.Contents.AddTag('OPTION') do begin
            Add(iif((
             (ItemIndex = i) and (FMultiSelect = False))
             or (Selected[i] and FMultiSelect)
             , 'SELECTED'));
            AddStringParam('VALUE', LValue);
            Contents.AddText(TextToHTML(LText));
          end;
        end;
      end
      else begin
        Result.Contents.AddText('');
      end;
    except FreeAndNil(Result); raise; end;
  end else begin
    with TIWLabel32.Create{$IFNDEF VSNET}(Self){$ENDIF} do try
      Name := Self.Name;
      ExtraTagParams.Assign(Self.ExtraTagParams);
      WebFont.Assign(Self.WebFont);
      Width := Self.Width;
      Height := Self.Height;
      Caption := '';
      RawText := FALSE; // Otherwise font info won't be rendered
      if Self.ItemIndex > -1 then begin
        if ItemsHaveValues then begin
          Caption := TextToHTMl(Self.Items.Values[Self.Items.Names[Self.ItemIndex]]);
        end else begin
          Caption := TextToHTML(Self.Items.Strings[Self.ItemIndex]);
        end;
      end;
      Result := RenderHTML(AContext);
    finally Free; end;
  end;
end;

procedure TIWCustomListbox32.ResetSelection;
begin
  FSelectedList.Clear;
  Invalidate;
end;

procedure TIWCustomListbox32.SetMultiSelect(const AValue: Boolean);
begin
  if AValue then begin
    FRequireSelection := False;
  end;
  FMultiSelect := AValue;
end;

procedure TIWCustomListbox32.SetSelected(AIndex: integer;
  const AValue: boolean);
begin
  if AValue then begin
    if not GetSelected(AIndex) then begin
      FSelectedList.Add(TObject(AIndex));
    end;
  end else begin
    FSelectedList.Remove(TObject(AIndex));
  end;
  Invalidate;
end;

procedure TIWCustomListbox32.SetValue(const AValue: string);
var
  LValue, s: string;
  i: Integer;
begin
  LValue := AValue;
  s := Fetch(LValue, ',');
  if ItemsHaveValues then begin
    for i := 0 to Items.Count - 1 do begin
      if s = Items.Values[Items.Names[i]] then begin
        ItemIndex := i;
        break;
      end;
    end;
  end else begin
    ItemIndex := StrToIntDef(s, -1);
  end;
  // For multiselect, first one is ItemIndex. Will be repeated in selected list
  // ItemIndex := StrToIntDef(Fetch(LValue, ','), -1);
  if FMultiSelect then begin
    ResetSelection;
    Selected[ItemIndex] := true;
    while Length(LValue) > 0 do begin
      s := Fetch(LValue, ',');
      if ItemsHaveValues then begin
        for i := 0 to Items.Count - 1 do begin
          if s = Items.Values[Items.Names[i]] then
            break;
        end;
      end else begin
        i := StrToInt(s);
      end;

      Selected[i] := True;
    end;
  end;
end;

procedure TIWCustomListbox32.InitItems;
begin
  inherited;
  FItems.OnChange := DoItemsChange;
  FItems.OnChanging := DoItemsChanging;
end;

{ TIWListbox32 }

procedure TIWListbox32.SetRequireSelection(const AValue: Boolean);
begin
  if AValue then begin
    FMultiSelect := False;
  end;
  inherited;
end;

end.

