unit IWCompMenu;

interface

uses
  {$IFDEF VSNET}
  {$ELSE}
  {$IFDEF Linux}QControls,{$ELSE}Controls,{$ENDIF}
  {$IFDEF Linux}Types, {$ELSE}Windows, {$ENDIF}
  {$IFDEF Linux}QMenus,{$ELSE}Menus,{$ENDIF}
  {$ENDIF}
  {$IFDEF Linux}QGraphics,{$ELSE}Graphics,{$ENDIF}
  Classes,
  IWAppForm, IWHTMLTag, IWControl, IWServerControllerBase,
  IWColor, IWFont, SWSystem, IWRenderContext, IWBaseHTMLInterfaces,
  IWBaseInterfaces;

type
  TIWMenu = class;

  TIWMenuOrientation = (iwOHorizontal, iwOVertical);

  TIWMenuStyle = class(TPersistent)
  private
    FSelectedItemBGColor: TIWColor;
    FItemBGColor: TIWColor;
    FDisabledItemBGColor: TIWColor;
    FBGColor: TIWColor;
    procedure SetDisabledItemBGColor(const Value: TIWColor);
    procedure SetItemBGColor(const Value: TIWColor);
    procedure SetSelectedItemBGColor(const Value: TIWColor);
    procedure SetBGColor(const Value: TIWColor);
  protected
    FColor: TIWColor;
    FSelectedMenuItemFont: TIWFont;
    FMenuItemFont: TIWFont;
    FMenuFont: TIWFont;
    FDisabledMenuItemFont : TIWFont;

    procedure SetMenuFont(const Value: TIWFont);
    procedure SetMenuItemFont(const Value: TIWFont);
    procedure SetSelectedMenuItemFont(const Value: TIWFont);
    procedure SetDisabledMenuItemFont(const Value: TIWFont);
  protected
    FIWMenu: TIWMenu;
    FOrientation: TIWMenuOrientation;

    procedure SetOrientation(const Value: TIWMenuOrientation);
  public
    constructor Create(AIWMenu: TIWMenu);

    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;
  published
    property MenuFont: TIWFont read FMenuFont write SetMenuFont;
    property MenuItemFont: TIWFont read FMenuItemFont write SetMenuItemFont;
    property SelectedMenuItemFont: TIWFont read FSelectedMenuItemFont write SetSelectedMenuItemFont;
    property DisabledMenuItemFont: TIWFont read FDisabledMenuItemFont write SetDisabledMenuItemFont;

    property BGColor: TIWColor read FBGColor write SetBGColor;
    property ItemBGColor: TIWColor read FItemBGColor write SetItemBGColor;
    property SelectedItemBGColor: TIWColor read FSelectedItemBGColor write SetSelectedItemBGColor;
    property DisabledItemBGColor: TIWColor read FDisabledItemBGColor write SetDisabledItemBGColor;

    property Orientation: TIWMenuOrientation read FOrientation write SetOrientation;
  end;

  TIWSpaceItems = (itsNone, itsEvenlySpaced);

  TIWMenuAutoSize = (mnaNone, mnaFullWidth, mnaFullHeight);

  TIWMenu = class(TIWCustomControl, IIWSubmitControl)
  private
    FComboBoxHide: Boolean;
  protected
    FSubmitParam : String;
    FAutoSize: TIWMenuAutoSize;
    FAttachedMenu: TMainMenu;
    FTimeOut: integer;
    FMenuLeft: integer;
    FMenuTop: integer;
    FPadding: integer;
    FTextOffset: integer;
    FItemSpacing: TIWSpaceItems;
    FFrameName: string;
    FMainMenuStyle: TIWMenuStyle;
    FSubMenuStyle: TIWMenuStyle;

    FMenuItems: TStringList;
    //
    procedure SetIWAutoSize(const Value: TIWMenuAutoSize);
    procedure SetMainMenuStyle(Value: TIWMenuStyle);
    procedure SetSubMenuStyle(Value: TIWMenuStyle);
    procedure SetAttachedMenu(Value: TMainMenu);
    procedure SetItemSpacing(Value: TIWSpaceItems);
    procedure SetPadding(Value: integer);
    procedure SetTextOffset(Value: integer);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

    procedure Submit(const AValue: string); override;
  public
    function GetSubmitParam : String;
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property TextOffset: integer read FTextOffset;
  published
    property FriendlyName;
    property AttachedMenu: TMainMenu read FAttachedMenu write SetAttachedMenu;
    property ItemSpacing: TIWSpaceItems read FItemSpacing write SetItemSpacing;
    property AutoSize: TIWMenuAutoSize read FAutoSize write SetIWAutoSize;
    property MainMenuStyle: TIWMenuStyle read FMainMenuStyle write SetMainMenuStyle;
    property SubMenuStyle: TIWMenuStyle read FSubMenuStyle write SetSubMenuStyle;
    property TimeOut: integer read FTimeOut write FTimeOut;
  private
    property ComboBoxHide: Boolean read FComboBoxHide write FComboBoxHide default true;
  end;
// Procs used during Paint

function FixCaption(Value: string): string;

implementation

uses
  {$IFDEF Linux}QForms,{$ELSE}Forms, {$ENDIF}
  IWApplication, SysUtils, IWServer, IWForm, IWBaseHTMLControl, IWImageList, IWGlobal,
  IWLayoutMgrForm;

type
  TCharSet = set of AnsiChar;

function TrimCharRight(Value: AnsiString; Chars: TCharSet): string;
begin
  if Value <> '' then
  begin
    while (Length(Value) > 0) and (Value[Length(Value)] in Chars) do
      delete(Value, Length(Value), 1);
  end;
  Result := Value;
end;

function TrimCharLeft(Value: AnsiString; Chars: TCharSet): string;
begin
  if Length(Value) > 0 then
  begin
    while (Length(Value) > 0) and (Value[1] in Chars) do
      delete(Value, 1, 1);
  end;
  Result := Value;
end;

function FixCaption(Value: string): string;
var
  i: integer;
begin
  Result := '';
  for i := 1 to length(value) do
  begin
    if value[i] <> '&' then
      Result := Result + Value[i];
  end;
end;

//
//	Make sure that the double quotes (") are correctly formatted
//

function DQuoteStr(const Value: string): string;
begin
  Result := '"' + TrimCharRight(TrimCharLeft(Value, ['''', '"']), ['''', '"']) + '"';
end;

{ TIWMenuStyle }

constructor TIWMenuStyle.Create;
begin
  inherited Create;
  // Set the Font defaults
  FIWMenu := AIWMenu;
  FSelectedMenuItemFont := TIWFont.Create;
  FMenuItemFont := TIWFont.Create;
  FMenuFont := TIWFont.Create;
  FDisabledMenuItemFont := TIWFont.Create;

  FBGColor := clWhite;
  FItemBGColor := clWhite;
  FSelectedItemBGColor := clSilver;
  FDisabledItemBGColor := clWhite;

  FDisabledMenuItemFont.Color := clLtGray;
end;

destructor TIWMenuStyle.Destroy;
begin
  FreeAndNil(FDisabledMenuItemFont);
  FreeAndNil(FSelectedMenuItemFont);
  FreeAndNil(FMenuItemFont);
  FreeAndNil(FMenuFont);

  inherited Destroy;
end;

procedure TIWMenuStyle.SetOrientation(const Value: TIWMenuOrientation);
begin
  FOrientation := Value;
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetMenuFont(const Value: TIWFont);
begin
  FMenuFont.Assign(Value);
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetMenuItemFont(const Value: TIWFont);
begin
  FMenuItemFont.Assign(Value);
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetSelectedMenuItemFont(const Value: TIWFont);
begin
  FSelectedMenuItemFont.Assign(Value);
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetDisabledMenuItemFont(const Value: TIWFont);
begin
  FDisabledMenuItemFont.Assign(Value);
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.Assign(Source: TPersistent);
begin
  if Source is TIWMenuStyle then begin
    with Source as TIWMenuStyle do begin
      Self.FSelectedMenuItemFont.Assign(SelectedMenuItemFont);
      Self.FColor := FColor;
      Self.FMenuItemFont.Assign(FMenuItemFont);
      Self.FMenuFont.Assign(FMenuFont);
      Self.FOrientation := FOrientation;
      Self.FIWMenu := FIWMenu;
      Self.FDisabledMenuItemFont.Assign(FDisabledMenuItemFont);
    end;
  end
  else
    inherited Assign(Source);
end;

procedure TIWMenuStyle.SetDisabledItemBGColor(const Value: TIWColor);
begin
  FDisabledItemBGColor := Value;
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetItemBGColor(const Value: TIWColor);
begin
  FItemBGColor := Value;
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetSelectedItemBGColor(const Value: TIWColor);
begin
  FSelectedItemBGColor := Value;
  FIWMenu.Invalidate;
end;

procedure TIWMenuStyle.SetBGColor(const Value: TIWColor);
begin
  FBGColor := Value;
  FIWMenu.Invalidate;
end;

{ TIWMenu }

constructor TIWMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FMainMenuStyle := TIWMenuStyle.Create(self);
  FSubMenuStyle := TIWMenuStyle.Create(self);

  FSubMenuStyle.Orientation := iwOVertical;

  FAutoSize := mnaNone;
  Height := 20;
  Width := 200;
  FTextOffset := 30;
  ItemSpacing := itsNone;

  FMenuItems := TStringList.Create;
  ComboBoxHide := true;

  SupportsPartial := false;
end;

destructor TIWMenu.Destroy;
begin
  FreeAndNil(FMainMenuStyle);
  FreeAndNil(FSubMenuStyle);
  FreeAndNil(FMenuItems);
  inherited;
end;

procedure TIWMenu.SetMainMenuStyle(Value: TIWMenuStyle);
begin
  FMainMenuStyle.Assign(Value);
  Invalidate;
end;

procedure TIWMenu.SetSubMenuStyle(Value: TIWMenuStyle);
begin
  FSubMenuStyle.Assign(Value);
  Invalidate;
end;

procedure TIWMenu.SetAttachedMenu(Value: TMainMenu);
begin
  FAttachedMenu := Value;
  Invalidate;
end;

procedure TIWMenu.SetItemSpacing(Value: TIWSpaceItems);
begin
  if FItemSpacing <> Value then
  begin
    FItemSpacing := Value;
    if FItemSpacing = itsNone then
      FAutoSize := mnaNone;

    Invalidate;
  end;
end;

procedure TIWMenu.SetPadding(Value: integer);
begin
  if FPadding <> Value then
  begin
    FPadding := Value;
    Invalidate;
  end;
end;

procedure TIWMenu.SetTextOffset(Value: integer);
begin
  if FTextOffset <> Value then
  begin
    FTextOffset := Value;
    Invalidate;
  end;
end;

//
//	THIS IS THE JAVASCRIPT/HTML VERSION OF THE CODE
//

Type
  TIWHTMLTable = class(TIWHTMLTag)
  protected
    FIWMenu: TIWMenu;
    FMenuItem: TMenuItem;
  public
    constructor Create(AMenu: TIWMenu; AMenuItem: TMenuItem); reintroduce;  
    function generateMenu(AContext: TIWComponent40Context; emptyElement: TIWHTMLTag; menuStyle: TIWMenuStyle; AIdent: Integer = 0): String;
    function generateMenuElement(AContext: TIWComponent40Context; AMenuItem: TMenuItem; menuStyle: TIWMenuStyle): TIWHTMLTag;
  end;

function TIWMenu.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  LScript: String;
  LMainMenuStyle: TIWMenuStyle;
begin
  Result := nil;
  set_RenderSize(AContext.ContainerContext.LayoutManager is TIWLayoutMgrForm); 
  if Assigned(FAttachedMenu) and (FAttachedMenu.Items.Count > 0) then begin

    (AContext as TIWComponent40Context).AddScriptFile('/js/IWMenu.js');

    // Result := TIWHTMLTag.CreateTag(''); try
      with (AContext.PageContext as TIWPageContext40).StyleTag do begin
        with MainMenuStyle.FMenuFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MainMenuStyle') then begin
            CSSStyle := 'MainMenuStyle';
            Contents.AddText(EOL + '.MainMenuStyle {');
            Contents.AddText('  text-align: center;');
            Contents.AddText('  border: 1px solid #000000;');
            if MainMenuStyle.BGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(MainMenuStyle.BGColor) + ';');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            Contents.AddText('}' + EOL);
          end;

        with MainMenuStyle.FSelectedMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MainMenuSelectedItemStyle') then begin
            CSSStyle := 'MainMenuSelectedItemStyle';
            Contents.AddText('.MainMenuSelectedItemStyle {');
            Contents.AddText('  text-align: center;');
            Contents.AddText('  border: 1px solid #000000;');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            if MainMenuStyle.SelectedItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(MainMenuStyle.SelectedItemBGColor) + ';');
            Contents.AddText('}' + EOL);
          end;

        with MainMenuStyle.FMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MainMenuItemStyle') then begin
            CSSStyle := 'MainMenuItemStyle';
            Contents.AddText('.MainMenuItemStyle {');
            Contents.AddText('  text-align: center;');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            if MainMenuStyle.ItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(MainMenuStyle.ItemBGColor) + ';');
            Contents.AddText('  border: 1px solid #FFFFFF;');
            Contents.AddText('}' + EOL);
          end;

        with MainMenuStyle.FDisabledMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MainDisabledMenuItemStyle') then begin
            CSSStyle := 'MainDisabledMenuItemStyle';
            Contents.AddText('.MainDisabledMenuItemStyle {');
            Contents.AddText('  text-align: center;');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            if MainMenuStyle.DisabledItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(MainMenuStyle.DisabledItemBGColor) + ';');
            Contents.AddText('  border: 1px solid #FFFFFF;');
            Contents.AddText('}' + EOL);
          end;

        with SubMenuStyle.FMenuFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MenuStyle') then begin
            CSSStyle := 'MenuStyle';
            Contents.AddText('.MenuStyle {');
            Contents.AddText('  text-align: left;');
            if SubMenuStyle.BGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(SubMenuStyle.BGColor) + ';');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            Contents.AddText('  border: 1px solid #000000;');
            Contents.AddText('}' + EOL);
          end;

        with SubMenuStyle.FSelectedMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MenuSelectedItemStyle') then begin
            CSSStyle := 'MenuSelectedItemStyle';
            Contents.AddText('.MenuSelectedItemStyle {');
            Contents.AddText('  text-align: left;');
            if SubMenuStyle.SelectedItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(SubMenuStyle.SelectedItemBGColor) + ';');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            Contents.AddText('  border: 1px solid #000000;');
            Contents.AddText('}' + EOL);
          end;

        with SubMenuStyle.FMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'MenuItemStyle') then begin
            CSSStyle := 'MenuItemStyle';
            Contents.AddText('.MenuItemStyle {');
            Contents.AddText('  text-align: left;');
            if SubMenuStyle.ItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(SubMenuStyle.ItemBGColor) + ';');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            Contents.AddText('  border: 1px solid #FFFFFF;');
            Contents.AddText('}' + EOL);
          end;

        with SubMenuStyle.FDisabledMenuItemFont do
          if (Trim(CSSStyle) = '') or (Trim(CSSStyle) = 'DisabledMenuItemStyle') then begin
            CSSStyle := 'DisabledMenuItemStyle';
            Contents.AddText('.DisabledMenuItemStyle {');
            Contents.AddText('  text-align: left;');
            if SubMenuStyle.DisabledItemBGColor <> clNone then
              Contents.AddText('  background-color:' + ColorToRGBString(SubMenuStyle.DisabledItemBGColor) + ';');
            Contents.AddText(FontToStringStyle(AContext.Browser));
            Contents.AddText('  border: 1px solid #FFFFFF;');
            Contents.AddText('}' + EOL);
          end;
      end;

      result := TIWHTMLTag.CreateTag(''); try
      // result := TIWHTMLTable.Create(self, AttachedMenu.Items); try;
      LMainMenuStyle := TIWMenuStyle.Create(self); try
        LMainMenuStyle.Assign(MainMenuStyle);
        FMenuItems.Clear;

        with result do begin // result.Contents.AddTag('div') do begin
        //  AddStringParam('id', HTMLName);
        //  AddStringParam('name', HTMLName);
        //  AddStringParam('class', HTMLName + 'CSS');

          FMenuItems.AddObject('', AttachedMenu.Items);
          // with TIWHTMLTable(result) do begin
          with TIWHTMLTable(Contents.AddTagAsObject(TIWHTMLTable.Create(self, AttachedMenu.Items))) do begin // First level menu
            // AddStringParam('width', '100%');
            // AddStringParam('height', '100%');
            LScript := generateMenu(TIWComponent40Context(AContext), result, LMainMenuStyle);
            AddStringParam('style', RenderStyle(AContext));
            // AddStringParam('class', HTMLName + 'CSS');
            AddStringParam('id', HTMLName);
          end;
        end;

        with TIWComponent40Context(AContext) do begin
          AddToJavaScriptOnce('var ' + HTMLName + '_menu;');

          LScript := HTMLName + '_menu = initMenu("' + HTMLName + {'_menu}'")' + EOL
            + 'with (' + HTMLName + '_menu) {' + EOL
            + '  SelectedItemStyle = "' + LMainMenuStyle.SelectedMenuItemFont.CSSStyle + '";' + EOL
            + '  MenuOrientation = ' +
              iif(LMainMenuStyle.Orientation = iwOHorizontal, 'moHorizontal',
              iif(LMainMenuStyle.Orientation = iwOVertical, 'moVertical', 'error'))
            + ';' + EOL
            + lScript + '}' + EOL;
         
          LScript := LScript + HTMLName + '_menu.ComboBoxHide = ' + iif(FComboBoxHide, 'true', 'false') + ';';

          CacheImage(HTMLName + '_arrow', WebApplication.InternalURLBase + '/gfx/menu_arrow_right.gif');
        end;
      finally
        LMainMenuStyle.Free;
      end;

      TIWComponent40Context(AContext).AddToInitProc(LScript);
    except FreeAndNil(Result); raise; end;
  end;
end;

procedure TIWMenu.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then begin
    if AComponent = FAttachedMenu then
    begin
      FAttachedMenu := nil;
      Invalidate;
    end;
  end;
end;

procedure TIWMenu.SetIWAutoSize(const Value: TIWMenuAutoSize);
begin
  FAutoSize := Value;
  if Value <> mnaNone then
    ItemSpacing := itsEvenlySpaced;
  Invalidate;
end;

procedure TIWMenu.Submit(const AValue: string);
begin
  FSubmitParam := AValue;
  TMenuItem(FMenuItems.Objects[FMenuItems.IndexOf(AValue)]).Click;
end;

function TIWMenu.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

{ TIWHTMLTable }

constructor TIWHTMLTable.Create(AMenu: TIWMenu; AMenuItem: TMenuItem);
begin
  inherited CreateTag('table');
  FIWMenu := AMenu;
  FMenuItem := AMenuItem;
end;

function TIWHTMLTable.generateMenuElement(AContext: TIWComponent40Context; AMenuItem: TMenuItem; menuStyle: TIWMenuStyle): TIWHTMLTag;
Var
  lImgList: TIWImageList;
  LPathName: String;
begin
  if AMenuItem.Enabled then begin
    AddStringParam('class', menuStyle.FMenuItemFont.CSSStyle);
  end else begin
    AddStringParam('class', menuStyle.FDisabledMenuItemFont.CSSStyle);
  end;
  AddIntegerParam('border', 0);
  AddStringParam('width', '100%');
  AddStringParam('height', '100%');
  AddIntegerParam('cellspacing', 0);
  AddIntegerParam('cellpadding', 0);
  AddStringParam('style', 'cursor:hand;');
  with FIWMenu do begin
    AddStringParam('id', HTMLName + '_submenu' + IntToStr(FMenuItems.IndexOfObject(AMenuItem)));
    FMenuItems[FMenuItems.IndexOfObject(AMenuItem)] :=
      HTMLName + '_submenu' + IntToStr(FMenuItems.IndexOfObject(AMenuItem));
  end;

  case menuStyle.Orientation of
    iwoHorizontal: begin
      with Contents.AddTag('tr') do begin
        with Contents.addtag('td') do begin
          AddStringParam('width', '16px');
          Contents.AddText('&nbsp;');
        end;
        
        with contents.AddTag('td') do begin
          Add('nowrap');
          Contents.AddText(StringReplace(AMenuItem.Caption, '&', '', [rfReplaceall]));
        end;

        with Contents.addtag('td') do begin
          AddStringParam('width', '16px');
          Contents.AddText('&nbsp;');
        end;
      end;
    end;
    iwOVertical: begin
      with contents.AddTag('tr') do begin
        with Contents.AddTag('td') do begin
          AddStringParam('width', '24px');
          AddStringParam('align', 'center');
          AddStringParam('valign', 'middle');
          // AddStringParam('class', menuprop.getMenuItem().getCSSStyle());
          if (FIWMenu.AttachedMenu.Images <> nil) and (AMenuItem.ImageIndex > -1) then begin
            lImgList := TIWImageList(FIWMenu.AttachedMenu.Images);
            LPathname := lImgList.ExtractImageToCache(AMenuItem.ImageIndex);
            if LPathname <> '' then begin
              with Contents.AddTag('img') do begin
                AddStringParam('src', GServerController.CacheURL + LPathname);
                AddIntegerParam('border', 0);
              end;
            end else begin
              contents.AddText('&nbsp;');
            end;
          end else begin
            contents.AddText('&nbsp;');
          end;
        end;

        with Contents.AddTag('td') do begin
          contents.AddText(StringReplace(AMenuItem.Caption, '&', '', [rfReplaceall]))
        end;

        with Contents.AddTag('td') do begin
          AddStringParam('width', '8px');
          contents.AddText('&nbsp;');
        end;

        (* with Contents.AddTag('td') do begin
          // ShortCutToText(AMenuItem.ShortCut)
          AddStringParam('align', 'right');
          contents.AddText(StringReplace(ShortCutToText(AMenuItem.ShortCut), '&', '', [rfReplaceall]))
        end; *)

        with Contents.AddTag('td') do begin
          AddStringParam('width', '16px');
          if AMenuItem.Count > 0 then begin
            with contents.AddTag('img') do begin
              AddStringParam('src', AContext.WebApplication.InternalURLBase + '/gfx/menu_arrow_right.gif');
            end;
          end else begin
            contents.AddText('&nbsp;')
          end;
        end;
      end;
    end;
  end;
  result := self;
end;

function TIWHTMLTable.generateMenu(AContext: TIWComponent40Context; emptyElement: TIWHTMLTag;
  menuStyle: TIWMenuStyle; AIdent: Integer = 0): String;
Var
  i: Integer;
  LIdent: String;
  tr: TIWHTMLTag;
  lMenuStyle: TIWMenuStyle;
begin
  result := '';
  tr := nil;

  // do_onMenuRender(m_Menu, context, newProp);

  AddStringParam('id', FIWMenu.HTMLName + '_submenu' + IntToStr(FIWMenu.FMenuItems.IndexOfObject(FMenuItem)) + '_table');
  FIWMenu.FMenuItems[FIWMenu.FMenuItems.IndexOfObject(FMenuItem)] :=
    FIWMenu.HTMLName + '_submenu' + IntToStr(FIWMenu.FMenuItems.IndexOfObject(FMenuItem)) + '_table';
  FIWMenu.FMenuItems[FIWMenu.FMenuItems.IndexOfObject(FMenuItem)] :=
    FIWMenu.HTMLName + '_submenu' + IntToStr(FIWMenu.FMenuItems.IndexOfObject(FMenuItem));    
  AddStringParam('class', menuStyle.FMenuFont.CSSStyle);
  AddIntegerParam('border', 0);
  AddIntegerParam('cellspacing', 0);
  AddIntegerParam('cellpadding', 1);
  AddStringParam('style', 'position:absolute; visibility:hidden; display:none; z-index:' + IntToStr(FIWMenu.ZIndex + 100));

  for i := 0 to FMenuItem.Count - 1 do begin
      if FMenuItem[i].Visible then begin
      if FMenuItem[i].IsLine then begin
        // the item is separator
        if menuStyle.Orientation = iwoVertical then begin
          Contents.AddTag('tr').Contents.AddTag('td').Contents.AddTag('hr');
        end;
      end else begin
        // the item is NOT separator
        FIWMenu.FMenuItems.AddObject('', FMenuItem[i]);
        if menuStyle.Orientation = iwoVertical then begin
          with contents.AddTag('tr').Contents.addtag('td') do
            TIWHTMLTable(Contents.AddTagAsObject(
              TIWHTMLTable.Create(FIWMenu, FMenuItem))).generateMenuElement(AContext, FMenuItem[i], menuStyle);
        end else begin
          if not assigned(tr) then begin
            tr := TIWHTMLTag.CreateTag('tr');
            contents.AddTagAsObject(tr);
          end;
          with tr.Contents.addtag('td') do
            TIWHTMLTable(Contents.AddTagAsObject(
              TIWHTMLTable.Create(FIWMenu, FMenuItem))).generateMenuElement(AContext, FMenuItem[i], menuStyle);
        end;
      end;
    end;
  end;

  if (FMenuItem = FIWMenu.AttachedMenu.Items) and (FIWMenu.FItemSpacing = itsNone) then begin
    if menuStyle.Orientation = iwoVertical then begin
      with contents.AddTag('tr').Contents.addtag('td') do begin
        AddStringParam('height', '100%');
        Contents.AddText('&nbsp;');
      end;
    end else begin
      if not assigned(tr) then begin
        tr := TIWHTMLTag.CreateTag('tr');
        contents.AddTagAsObject(tr);
      end;
      with tr.Contents.addtag('td') do begin
        AddStringParam('width', '100%');
        Contents.AddText('&nbsp;');
      end;
    end;
  end;

  LIdent := StringOfChar(' ', AIdent * 2);

  for i := 0 to FMenuItem.Count - 1 do begin
    if FMenuItem[i].Visible then begin
      if not FMenuItem[i].IsLine then begin
        Result := Result + LIdent + 'with (addMenuItem("' + FIWMenu.HTMLName + '_submenu'
              + IntToStr(FIWMenu.FMenuItems.IndexOfObject(FMenuItem[i])) + '", id)) { ' + EOL
              + LIdent + '  SelectedItemStyle = "' + menuStyle.SelectedMenuItemFont.CSSStyle + '";'
              + EOL;

        if FMenuItem[i].Count > 0 then begin
          with TIWHTMLTable(emptyElement.Contents.AddTagAsObject(TIWHTMLTable.Create(FIWMenu, FMenuItem[i]))) do begin
            lMenuStyle := TIWMenuStyle.Create(FIWMenu); try
              lMenuStyle.Assign(FIWMenu.FSubMenuStyle);
              result := result + generateMenu(AContext, emptyElement, lMenuStyle, AIdent + 1);
              result := result + LIdent + '  MenuOrientation = '
                + iif(lMenuStyle.Orientation = iwOHorizontal, 'moHorizontal',
                  iif(lMenuStyle.Orientation = iwOVertical, 'moVertical', 'error'))
                +  ';' + EOL;
            finally
              lMenuStyle.Free;
            end;
          end;
        end;

        result := result + LIdent + '}' + EOL;
      end;
    end;
  end;
end;

initialization
  TIWServer.AddInternalFile('IW_JS_IWMENU', '/js/IWMenu.js');
  TIWServer.AddInternalFile('IW_GFX_menu_arrow_right', '/gfx/menu_arrow_right.gif');
end.



