unit IWExtCtrls;
{PUBDIST}

interface

uses
  {$IFDEF VSNET}
  System.ComponentModel, System.Drawing, System.IO,
  IWNetBaseControl,
  IWNetClasses,
  System.Drawing.Design, System.Drawing.Imaging,
  {$ELSE}
  Classes,
    {$IFDEF Linux}
    IWCLXComponent, IWCLXClasses,
    {$ELSE}
    IWVCLComponent, IWVCLClasses,
    {$ENDIF}
  {$ENDIF}
  {$IFDEF Linux}QGraphics, {$ELSE}Graphics, {$ENDIF}
  SysUtils,
  IWBaseHTMLControl, IWControl, IWTypes, IWFileReference, IWHTMLTag,
  IWScriptEvents, IWGlobal, IWColor, IWRenderContext, IWJpegOptions,
  IWContainerBorderOptions, IWBaseInterfaces, IWHTML40Interfaces,
  IWBaseHTMLComponent, IWBaseRenderContext;

type
  TIWCustomRadioGroup = class(TIWCustomControl, IIWInputControl, IIWSubmitControl)
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  protected
    FSubmitParam : String;  
    FItemIndex: Integer;
    FItems: TIWStringList;
    FLayout: TIWRadioGroupLayout;
    //
    procedure SetItemIndex(AIndex: Integer);
    procedure OnItemsChange( ASender : TObject );
    procedure SetItems(AValue: TIWStringList);
    procedure SetLayout(const AValue: TIWRadioGroupLayout);
    procedure SetValue(const AValue: string); virtual;
    procedure HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents); override;
    procedure Submit(const AValue: string); override;
    procedure GetInputControlNames(ANames: TStringList); override;
    function IsForThisControl(AName: string): boolean; override;

    procedure InitControl; override;
    procedure DoClick; override;
  public
    function GetSubmitParam : String;  
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
    function RenderStyle(AContext: TIWBaseHTMLComponentContext): string; override;

    property ItemIndex: integer read FItemIndex write SetItemIndex;
    {$IFDEF VSNET}
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    {$ENDIF}
    property Items: TIWStringList read FItems write SetItems;

    property Layout: TIWRadioGroupLayout read FLayout write SetLayout;
  published
    // {$IFNDEF VSNET}
    property OnClick; //: TNotifyEvent read GetOnClick write SetOnClick;
    // {$ENDIF}
    property Confirmation;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWExtCtrls.TIWRadioGroup.bmp}
  TIWRadioGroup = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWRadioGroup), 'TIWRadioGroup.bmp')]
  {$ENDIF}
  TIWRadioGroup = class(TIWCustomRadioGroup)
  published
    property Editable;
    property Enabled;
    {$IFDEF CLR}
    property WebFont;
    {$ELSE}
    property Font;
    {$ENDIF}
    property FriendlyName;
    property ItemIndex;
    property Items;
    property Layout;
    property TabOrder;
  end;

  TIWImageOnMouseDown = procedure(ASender: TObject; const AX: Integer; const AY: Integer) of object;

  TIWCustomImage = class(TIWCustomControl, IIWSubmitControl)
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  protected
    FSubmitParam : String;
    FAltText: string;
    FBorderOptions: TIWBorderOptions;
    FOnMouseDown: TIWImageOnMouseDown;
    FPicture: TIWPicture;
    FUseSize: Boolean;
    {$IFDEF VSNET}
    FImage: Image;
    {$ENDIF}
    //
    function CanAutoSize(var VNewWidth: Integer;
      var VNewHeight: Integer): Boolean; {$IFNDEF Linux} override; {$ENDIF}
    procedure CreatePicture;
    // Necessary because Borland made FOnClick protected
    // {$IFNDEF VSNET}
    function GetOnClick: {$IFDEF VSNET}EventHandler {$ELSE} TNotifyEvent{$ENDIF};
    procedure SetOnClick(const AValue: {$IFDEF VSNET}EventHandler {$ELSE} TNotifyEvent{$ENDIF});
    // {$ENDIF}
    function ImageHTML(AContext: TIWBaseHTMLComponentContext; const ASrc: string): TIWHTMLTag;
    procedure HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents); override;
    procedure PictureChanged(ASender: TObject);
    procedure SetOnMouseDown(const AValue: TIWImageOnMouseDown);
    procedure SetPicture(AValue: TIWPicture);
    function GetPicture: TIWPicture;
    {$IFDEF VSNET}
    procedure SetImage(AValue: Image);
    {$ENDIF}
    procedure Submit(const AValue: string); override;
    procedure SetUseSize(const AValue: Boolean);
    procedure SetBorderOptions(AValue: TIWBorderOptions);

    procedure SetAutoSize(AValue: Boolean); override;

    procedure InitControl; override;
    {$IFDEF VSNET}
    property Image: Image read FImage write SetImage;
    {$ENDIF}
  public
    function GetSubmitParam : String;
    property Picture: TIWPicture read GetPicture write SetPicture;
  published
    property AltText: string read FAltText write FAltText;
{$IFNDEF Linux}
    // Necessary as Delphi defaults to False and causes streaming problems
    property AutoSize default True;
{$ENDIF}

    property BorderOptions: TIWBorderOptions read FBorderOptions write SetBorderOptions;
    property Confirmation;
    property DoSubmitValidation;
    property ExtraTagParams;
    property ScriptEvents;

    property TabOrder;

    property UseSize: Boolean read FUseSize write SetUseSize;
    //
    // Events
    //
    // {$IFNDEF VSNET}
    property OnClick: {$IFDEF VSNET}EventHandler {$ELSE} TNotifyEvent{$ENDIF} read GetOnClick write SetOnClick;
    // {$ENDIF}
    property OnMouseDown: TIWImageOnMouseDown read FOnMouseDown write SetOnMouseDown;
  end;

  TIWDynamicImage = class(TIWCustomImage)
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  protected
    FJpegOptions: TIWJpegOptions;
    //
    procedure SetJpegOptions(const AValue: TIWJpegOptions);

    procedure InitControl; override;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
  published

    property JpegOptions: TIWJpegOptions read FJpegOptions write SetJpegOptions;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWExtCtrls.TIWImage.bmp}
  TIWImage = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWImage), 'TIWImage.bmp')]
  {$ENDIF}
  TIWImage = class(TIWDynamicImage)
  published
    {$IFDEF VSNET}
    property Image;
    {$ELSE}
    property Picture;
    {$ENDIF}
    property FriendlyName;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWExtCtrls.TIWImageFile.bmp}
  TIWImageFile = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWImageFile), 'TIWImageFile.bmp')]
  {$ENDIF}
  TIWImageFile = class(TIWCustomImage)
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  protected
    FImageFile: TIWFileReference;
    //
    procedure ReferenceChanged(ASender: TObject);
    procedure SetImageFile(const AValue: TIWFileReference);

    procedure Loaded; override;

    procedure InitControl; override;
  public
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
  published
    property FriendlyName;
    {$IFDEF VSNET}
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    {$ENDIF}
    property ImageFile: TIWFileReference read FImageFile write SetImageFile;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWExtCtrls.TIWTimer.bmp}
  TIWTimer = class;
  [
    ToolboxItem(true),
    ToolboxBitmap(typeof(TIWTimer), 'TIWTimer.bmp'),
    ToolboxItemFilter('IWAppForm.TIWAppForm'),
    TIWToolPalette('Intraweb Controls')
  ]
  {$ENDIF}
  TIWTimer = class(TIWBaseHTMLComponent, IIWHTML40Component, IIWSubmitControl)
  protected
    FSubmitParam : String;  
    FEnabled: Boolean;
    FInterval: Integer;
    FOnTimer: TNotifyEvent;
    //
    procedure Submit(const AValue: string); virtual;
    function GetSubmitProc: TSubmitProc;

    function get_ControlEncode: TIWControlEncode;
    procedure SetControlEncode(const AValue: TIWControlEncode);

    function ComponentContextClass: TIWBaseComponentContextClass; override;

    procedure InitControl; override;

    function get_DoRefreshControl: Boolean;
    procedure set_DoRefreshControl(AValue: Boolean);

    function RenderCSSClass(AComponentContext: TIWBaseHTMLComponentContext): string; virtual;
    function RenderStyle(AComponentContext: TIWBaseHTMLComponentContext): string; virtual;

    procedure RenderScripts(AComponentContext: TIWBaseHTMLComponentContext); virtual;

    function WantsSubmitValidation: Boolean;

    property ControlEncode: TIWControlEncode read get_ControlEncode write SetControlEncode;
  public
    function GetSubmitParam : String;
    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;
  published
    property Enabled: Boolean read FEnabled write FEnabled;
    property Interval: Integer read FInterval write FInterval;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

implementation

uses
  {$IFDEF VSNET}
  Classes, IWNetComponent,
  {$ELSE}
  {$IFDEF Linux}QControls,{$ELSE}Controls,{$ENDIF}
  {$ENDIF}
  IWServerControllerBase,
  SWSystem, SWStrings;

{ TIWRadioGroup }

procedure TIWCustomRadioGroup.InitControl;
begin
  inherited;
  FItems := TIWStringList.Create;
  TIWStringList(FItems).OnChange := OnItemsChange;  
  FNeedsFormTag := True;
  Width := 89;
  Height := 112;
end;

procedure TIWCustomRadioGroup.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FItems);
  inherited;
end;

procedure TIWCustomRadioGroup.Submit(const AValue: string);
begin
  FSubmitParam := AValue;
 { DoClick; { Call of DoClick is commented to prevent it to happen on every click
              because Delphi users are accostomed that OnClick is fire only when
              the value is changed. 
              Ivan }
end;

procedure TIWCustomRadioGroup.HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents);
begin
  inherited HookEvents(AContext, AScriptEvents);
  if Editable then begin
    AScriptEvents.HookEvent('OnClick', iif(HasOnClick, SubmitHandler));
  end;
end;

function TIWCustomRadioGroup.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

function TIWCustomRadioGroup.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  i: Integer;
begin
  // To identify them as a group, HTML requires they all have the same name
  Result := TIWHTMLTag.CreateTag('DIV'); try
    for i := 0 to Items.Count - 1 do begin
      with Result.Contents.AddTag('INPUT') do begin
        AddStringParam('TYPE', 'RADIO');
        Add(IIF(ItemIndex = i, ' CHECKED'));
        AddStringParam('NAME', HTMLName + '_INPUT');
        // AddStringParam('id', HTMLName + '_INPUT');
        AddStringParam('VALUE', IntToStr(i));
        Add(IIF((not Enabled) or (not Editable) , 'DISABLED'));
      end;
      Result.Contents.AddText(Items.Strings[i]);
      Result.Contents.AddText(iif(Layout = glVertical, '<BR>', ' '));
    end;
  except FreeAndNil(Result); raise; end;
end;

function TIWCustomRadioGroup.RenderStyle(AContext: TIWBaseHTMLComponentContext): string;
begin
  Result := inherited RenderStyle(AContext);
  Result := Result + iif(FLayout = glHorizontal, ' white-space:nowrap;', '');
end;

procedure TIWCustomRadioGroup.OnItemsChange( ASender : TObject );
begin
  DoRefreshControl := True;
  Invalidate;
end;

procedure TIWCustomRadioGroup.SetItemIndex(AIndex: Integer);
begin
  if AIndex <> FItemIndex then begin
    FItemIndex := AIndex;
  DoRefreshControl := True;
  end;
end;

procedure TIWCustomRadioGroup.SetItems(AValue: TIWStringList);
begin
  FItems.Assign(AValue);
  Invalidate;
end;

procedure TIWCustomRadioGroup.SetLayout(const AValue: TIWRadioGroupLayout);
begin
  FLayout := AValue;
  Invalidate;
end;

procedure TIWCustomRadioGroup.DoClick;
begin
  { Inherited method is not called here because it will cause OnClick
    to be fired. However in this case OnClick will be fired always when
    it CheckBox is clicked even if the value is not changed (HTML feature)
    but Delphi users are used that IWCustomRadioGroup's OnClick event means that the value
    is changed. In TIWCustomRadioGroup, OnClick is called in SetValue proc.
    Ivan.
    }
//  inherited;
end;

procedure TIWCustomRadioGroup.SetValue(const AValue: string);
var
  LNewIndex : Integer;
begin
  LNewIndex := StrToIntDef(AValue, -1);
  if ItemIndex <> LNewIndex then begin
    ItemIndex := LNewIndex;
    Invalidate;
    inherited DoClick;
  end; 
end; 

{ TIWCustomImage }

function TIWCustomImage.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

procedure TIWCustomImage.SetAutoSize(AValue: Boolean);
begin
  inherited;

  if AutoSize then begin
    UseSize := false;
    if Assigned(FPicture) then begin
      PictureChanged(nil);
    end;
  end;
end;

function TIWCustomImage.CanAutoSize(var VNewWidth, VNewHeight: Integer): Boolean;
begin
  Result := False;
  // FPicture is nil during initial create at design time for TIWImageFile (inherited Create)
  // Seems wierd to check AutoSize esp during streaming, but its required otehrwise during
  // AutoSize = False and streaming at runtime we get witdh and height of 0.
  if (FPicture <> nil) and (IsDesignMode = False) and (AutoSize = True) then begin
    if (FPicture.Width > 0) and (FPicture.Height > 0) then begin
      Result := True;
      if Align in [alNone, alLeft, alRight] then begin
        VNewWidth := FPicture.Width;
      end;
      if Align in [alNone, alTop, alBottom] then begin
        VNewHeight := FPicture.Height;
      end;
    end;
  end;
end;

procedure TIWCustomImage.InitControl; 
begin
  inherited;
  AutoSize := True;
  Width := 89;
  Height := 112;
  FBorderOptions := IWContainerBorderOptions.TIWBorderOptions.Create;
  FBorderOptions.Control := Self;
end;

procedure TIWCustomImage.CreatePicture;
begin
  FPicture := TIWPicture.Create;
  FPicture.OnChange := PictureChanged;
end;

procedure TIWCustomImage.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FPicture);
  FreeAndNil(FBorderOptions);
  inherited;
end;

// {$IFNDEF VSNET}
function TIWCustomImage.GetOnClick:{$IFDEF VSNET}EventHandler {$ELSE} TNotifyEvent{$ENDIF};
begin
  Result := (inherited OnClick);
end;

procedure TIWCustomImage.SetOnClick(const AValue: {$IFDEF VSNET}EventHandler {$ELSE} TNotifyEvent{$ENDIF});
begin
  FOnMouseDown := nil;
  inherited OnClick := AValue;
end;
//{$ENDIF}

function TIWCustomImage.ImageHTML(AContext: TIWBaseHTMLComponentContext; const ASrc: string): TIWHTMLTag;
var
  LTag: TIWHTMLTag;
begin
  Result := TIWHTMLTag.CreateTag('IMG'); try
    Result.AddStringParam('SRC', ASrc);
    Result.AddStringParam('NAME', HTMLName);
    Result.AddIntegerParam('BORDER', BorderOptions.Width);
    if toTColor(BorderOptions.Color) <> clNone then begin
      Result.Add('style=border-color: ' + ColorToRGBString(BorderOptions.Color));
    end;
    Result.AddStringParam('ALT', AltText);
    if UseSize then begin
      Result.AddIntegerParam('WIDTH', Width);
      Result.AddIntegerParam('HEIGHT', Height);
    end;

    if Assigned(OnMouseDown) then begin
      if AContext.Browser in [brNetscape6, brNetscape7] then begin
        LTag := TIWHTMLTag.CreateTag('A'); try
          LTag.AddStringParam('HREF', '#');
          LTag.AddStringParam('ID', HTMLName + '_IMAGE');
          LTag.AddStringParam('OnMouseOver', 'return ImageSetEvent(this,''' + HTMLName + ''''
            + ',' + iif(DoSubmitValidation, 'true', 'false')
            + ',' + '''' + TextToJSStringLiteral(Confirmation) + ''''
            + ');"');
          LTag.Contents.AddTagAsObject(Result, true);
        except FreeAndNil(LTag); raise; end;
      end;
    end;
    // OnMouseDown and OnClick are mutually exclusive, no need to check
    // TODO: Fix for .NET
    if HasOnClick then begin
      LTag := SubmitLink('', '');
      LTag.Contents.AddTagAsObject(result, true);
      // result := ltag;
    end;
  except FreeAndNil(Result); raise; end;
end;

procedure TIWCustomImage.PictureChanged(ASender: TObject);
begin
  if not IsLoading then begin
    if AutoSize and (FPicture.Width > 0) and (FPicture.Height > 0) then begin
      SetBounds(Left, Top, FPicture.Width, FPicture.Height);
    end;
    Invalidate;
  end;
end;

procedure TIWCustomImage.SetOnMouseDown(const AValue: TIWImageOnMouseDown);
begin
  inherited OnClick := nil;
  FOnMouseDown := AValue;
end;

procedure TIWCustomImage.Submit(const AValue: string);
var
  s: string;
  LX: Integer;
  LY: Integer;
begin
  FSubmitParam := AValue;
  if Assigned(OnMouseDown) then begin
    s := AValue;
    LX := StrToInt(Fetch(s, ','));
    LY := StrToInt(s);
    OnMouseDown(Self, LX, LY);
  end else begin
    DoClick;
  end;
end;

procedure TIWCustomImage.SetUseSize(const AValue: Boolean);
begin
  set_RenderSize(AValue);
  FUseSize := AValue;
  if FUseSize then
    AutoSize := false;
end;

procedure TIWCustomImage.SetBorderOptions(AValue: TIWBorderOptions);
begin
  FBorderOptions.Assign(AValue);
end;

procedure TIWCustomImage.HookEvents(AContext: TIWPageContext40; AScriptEvents: TIWScriptEvents);
begin
  inherited;
  if Assigned(OnMouseDown) then begin
    if AContext.WebApplication.Browser in [brIE, brOpera] then begin
      AScriptEvents.HookEvent('OnMouseDown', 'return ImageClick(this.name,event,'
        + iif(DoSubmitValidation, 'true', 'false') + ', ''' + TextToJSStringLiteral(Confirmation) + ''');');
    end;
  end;
end;

{$IFDEF VSNET}
procedure TIWCustomImage.SetImage(AValue: Image);
Var
  ms: MemoryStream;
  ms1: TCLRStreamWrapper;
begin
  FImage := AValue;
  if Assigned(FImage) then begin
    ms := MemoryStream.Create; try
      FImage.Save(ms, ImageFormat.Bmp);
      ms1 := TCLRStreamWrapper.Create(ms); try
        Picture.Graphic := TBitmap.Create;
        ms1.Position := 0;
        Picture.Graphic.LoadFromStream(ms1);
      finally
        ms1.Free;
      end;
    finally
      ms.Free;
    end;
  end else begin
    Picture.Graphic := nil;
  end;
end;
{$ENDIF}

function TIWCustomImage.GetPicture: TIWPicture;
begin
  if {IsDesignMode and }not Assigned(FPicture) then begin
    CreatePicture;
  end;
  result := FPicture;
end;

procedure TIWCustomImage.SetPicture(AValue: TIWPicture);
begin
  Picture.Assign(AValue);
  // No need for Invalidate, Picture.OnChange will catch it
end;

{ TIWImage }

procedure TIWDynamicImage.InitControl;
begin
  inherited;
  FJpegOptions := IWJpegOptions.TIWJpegOptions.Create;
end;

procedure TIWDynamicImage.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FJpegOptions);
  inherited;
end;

function TIWDynamicImage.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
var
  LPathname: string;
begin
  LPathname := TIWServerControllerBase.NewCacheFile('jpg');
  // Is nil if no picture. Ex: Drop a TIWImage on form, dont load an image, run.
  if Picture.Graphic <> nil then begin
    if not FPicture.Graphic.Empty then begin
      // TODO: Fix for .NET
      ToJPegFile(FPicture.Graphic, LPathname, FJPegOptions);
      Result := ImageHTML(AContext, GServerController.CacheURL + ExtractFilename(LPathname));
    end else begin
      result := TIWHTMLTag.CreateTag('span');
    end;
  end else begin
    result := TIWHTMLTag.CreateTag('span');
  end;
end;

procedure TIWDynamicImage.SetJpegOptions(const AValue: TIWJpegOptions);
begin
  FJpegOptions.Assign(AValue);
end;

{ TIWImageFile }

procedure TIWImageFile.InitControl;
begin
  inherited;
  FImageFile := TIWFileReference.Create;
  FImageFile.OnChange := ReferenceChanged;
end;

procedure TIWImageFile.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FImageFile);
  inherited;
end;

procedure TIWImageFile.ReferenceChanged(ASender: TObject);
var
  LBitmap : TBitmap;
begin
  if IsDesignMode then
  begin
    if FileExists(ImageFile.Filename) then
    begin
      try
        Picture.LoadFromFile(ImageFile.Filename);
      except
        on EInvalidGraphic do
        begin
          LBitmap := TBitmap.Create;
          try
            LBitmap.Width := Width;
            LBitmap.Height := Height;
            LBitmap.Canvas.Font.Color := clRed;
            LBitmap.Canvas.TextOut( 5, 5, 'Please see the FAQ about' );
            LBitmap.Canvas.TextOut( 5, 20, 'enabling GIF support' );
            LBitmap.Canvas.TextOut( 5, 35, 'at design time' );
            Picture.Bitmap := LBitmap;
          finally
            FreeAndNil( LBitmap );
          end;
        end;
      end;
    end
    else
    begin
      Picture.Graphic := nil;
    end;
    Invalidate;
  end
  else
  begin
    if AutoSize and (FPicture <> nil) then
    begin
      if (FPicture.Graphic.Width > 0) and (FPicture.Graphic.Height > 0) then
      begin
        Width := FPicture.Graphic.Width;
        Height := FPicture.Graphic.Height;
      end;
    end;
    DoRefreshControl := TRUE;
  end;
end;

function TIWImageFile.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
begin
  Result := ImageHTML(AContext, ImageFile.Location(GServerController.FilesURL));
end;

procedure TIWImageFile.Loaded;
begin
  inherited;

  if IsDesignMode then begin
    if AutoSize and (Picture.Width > 0) and (Picture.Height > 0) then begin
      SetBounds(Left, Top, Picture.Width, Picture.Height);
    end;
    Invalidate;
  end;
end;

procedure TIWImageFile.SetImageFile(const AValue: TIWFileReference);
begin
  // Assign calls DoChange
  FImageFile.Assign(AValue);
end;

{ TIWTimer }

procedure TIWTimer.InitControl;
begin
  inherited;
  FEnabled := true;
  Interval := 1000;
end;

function TIWTimer.get_ControlEncode: TIWControlEncode;
begin
  result := ceNone;
end;

function TIWTimer.GetSubmitParam: String;
begin
  Result := FSubmitParam;
end;

function TIWTimer.GetSubmitProc: TSubmitProc;
begin
  result := Submit;
end;

function TIWTimer.ComponentContextClass: TIWBaseComponentContextClass;
begin
  result := TIWComponent40Context;
end;

function TIWTimer.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
begin
  result := nil;
end;

procedure TIWTimer.SetControlEncode(const AValue: TIWControlEncode);
begin
  // 
end;

procedure TIWTimer.Submit(const AValue: string);
begin
  FSubmitParam := AValue;
  if Assigned(OnTimer) then begin
    OnTimer(Self);
  end;
end;

function TIWTimer.get_DoRefreshControl: Boolean;
begin
  result := false;
end;

procedure TIWTimer.set_DoRefreshControl(AValue: Boolean);
begin
  // 
end;

function TIWTimer.RenderCSSClass(AComponentContext: TIWBaseHTMLComponentContext): string;
begin
  result := '';
end;

function TIWTimer.RenderStyle(AComponentContext: TIWBaseHTMLComponentContext): string;
begin
  result := '';
end;

procedure TIWTimer.RenderScripts(AComponentContext: TIWBaseHTMLComponentContext); 
var
  s : string;
  LContext : TIWComponent40Context;
begin
  LContext := TIWComponent40Context(AComponentContext);
  if Enabled and (Interval > 0) then begin
    // timer is enabled
    s := 'top.' + HTMLName + '_ID=setInterval(''top.SubmitClick("' + HTMLName
        + '","", false)'',' + IntToStr(Interval) + ');' + EOL;

    if LContext.WebApplication.IsPartialUpdate then begin
      LContext.AddToUpdateInitProc( s );
    end else begin
      LContext.AddToInitProc( s );
    end;
  end else begin
    // not enabled
    if LContext.WebApplication.IsPartialUpdate then begin
      LContext.AddToUpdateInitProc(
        'if(top.' + HTMLName + '_ID){clearInterval(top.' + HTMLName + '_ID);};' + EOL );
    end else begin
      LContext.AddToInitProc('top.' + HTMLName + '_ID=null;' + EOL );
    end;
  end;
end;

function TIWTimer.WantsSubmitValidation: Boolean;
begin
  result := false;
end;

procedure TIWCustomRadioGroup.GetInputControlNames(ANames: TStringList);
begin
  ANames.Add(HTMLName + '_INPUT')
end;

function TIWCustomRadioGroup.IsForThisControl(AName: string): boolean;
begin
  result := AName = HTMLName + '_INPUT';
end;

end.
