unit IWHTMLTag;
{PUBDIST}

interface

uses
  Classes,
  IWColor, IWMarkupLanguageTag;

type
  TIWHTMLTag = class;

  TIWHTMLTagCollection = class(TIWMarkupLanguageTagCollection)
    function AddTagAsObject(const ATag: TIWHTMLTag;
      const ASetParentElement: Boolean = false): TIWHTMLTag; overload;
    function AddTag(const ATag: string; const ASetParentElement: Boolean = false): TIWHTMLTag; reintroduce;
  end;

  TIWHTMLTag = class(TIWMarkupLanguageTag)
  private
    function GetContents: TIWHTMLTagCollection;
  protected
    procedure SetTag(const Value: AnsiString); override;

    function ContentsClass: TIWMarkupLanguageTagCollectionClass; override;
  public
    constructor CreateHTMLTag(const ATag: AnsiString; const AClosingTag: TIWCloseTag = cbTrue;
      const AParentElement: TIWMarkupLanguageElement = nil);

    property ClosingTag;
    property Contents: TIWHTMLTagCollection read GetContents;
  end;

  TIWHTMLTagClass = class of TIWHTMLTag;

implementation

uses
  {$IFDEF Linux}QGraphics,{$ELSE}Graphics,{$ENDIF}
  SysUtils,
  SWSystem;

const
  NeverClosedTags: String = 'area,br,frame,hr,img,input,link';

Var
  GClosingTagList: TStringList;


{ TIWHTMLTag }

function TIWHTMLTag.ContentsClass: TIWMarkupLanguageTagCollectionClass;
begin
  result := TIWHTMLTagCollection;
end;

constructor TIWHTMLTag.CreateHTMLTag(const ATag: AnsiString;
  const AClosingTag: TIWCloseTag;
  const AParentElement: TIWMarkupLanguageElement);
begin
  inherited Create(AParentElement);
  FClosingTag := AClosingTag;
  Tag := ATag;
end;

function TIWHTMLTag.GetContents: TIWHTMLTagCollection;
begin
  result := TIWHTMLTagCollection(inherited Contents);
end;

procedure TIWHTMLTag.SetTag(const Value: AnsiString);
begin
  inherited;

  if (GClosingTagList.IndexOf(Tag) > -1) or (FTag = '') then begin
    FClosingTag := cbFalse;
  end;

  if AnsiSameText(AnsiString('PRE'), Tag) then begin
    IsPreformated := true;
  end;

  if AnsiSameText(AnsiString('A'), Tag) then begin
    IsPreformated := true;
  end;

  if AnsiSameText(AnsiString('DIV'), Tag) then begin
    IsPreformated := true;
  end;
end;

{ TIWHTMLTagCollection }

function TIWHTMLTagCollection.AddTag(const ATag: string;
  const ASetParentElement: Boolean): TIWHTMLTag;
begin
  result := TIWHTMLTag(inherited AddTag(ATag, ASetParentElement));
end;

function TIWHTMLTagCollection.AddTagAsObject(const ATag: TIWHTMLTag;
  const ASetParentElement: Boolean): TIWHTMLTag;
begin
  result := TIWHTMLTag(inherited AddTagAsObject(ATag, ASetParentElement));
end;

initialization
  GClosingTagList := TStringList.Create;
  GClosingTagList.CommaText := NeverClosedTags;
finalization
  GClosingTagList.Free;
end.
