unit IWMain;

interface

{$IFNDEF CLR}
  {$DENYPACKAGEUNIT ON}
{$ENDIF}

uses
  {$IFDEF CLR}
  System.Resources, System.Reflection, System.Drawing, SWSystem,
  {$ENDIF}
  {$IFDEF Linux}QActnList,{$ELSE}ActnList,{$ENDIF}
  {$IFDEF Linux}QControls,{$ELSE}Controls,{$ENDIF}
  {$IFDEF Linux}QComCtrls,{$ELSE}ComCtrls,{$ENDIF}
  {$IFDEF Linux}QForms,{$ELSE}Forms,{$ENDIF}
  {$IFDEF Linux}QGraphics,{$ELSE}Graphics,{$ENDIF}
  {$IFDEF Linux}QStdCtrls,{$ELSE}StdCtrls,{$ENDIF}
  {$IFDEF Linux}QExtCtrls,{$ELSE}ExtCtrls,{$ENDIF}
  {$IFDEF Linux}QDialogs,{$ELSE}Dialogs,{$ENDIF}
  {$IFDEF Linux}QClipbrd,{$ELSE}Clipbrd,{$ENDIF}
  {$IFDEF Linux}Types,{$ELSE}Windows, Messages,{$ENDIF}
  Classes,
  IWHTTPServer, IWStandAloneServer,
  SysUtils, SyncObjs;
{$IFDEF Linux}
(*$HPPEMIT '#pragma link "IWMain.o"'*)
{$ELSE}
(*$HPPEMIT '#pragma link "IWMain.obj"'*)
{$ENDIF}

type
  TFormIWMain = class(TForm)
  protected
    FActionList: TActionList;
    FActionFile_ClearSessions: TAction;
    FActionFile_CopyURLToClipboard: TAction;
    FActionFile_Exit: TAction;
    FActionHelp_About: TAction;
    FActionRun_Execute: TAction;
    FActionRun_ExecuteIE: TAction;
    FActionRun_ExecuteDefault: TAction;
    FActionRun_ExecuteMozilla: TAction;
    FActionRun_ExecuteNS6: TAction;
    FActionRun_ExecuteOpera: TAction;
    FActionOptions_ExecuteSSL: TAction;
    FActionOptions_ShowDebugInformation: TAction;
    FCreateWndCalled: Boolean;
    FMemoLog: TMemo;
    FStatusBar: TStatusBar;
    lablActiveSessions: TLabel;
    lablConnectionSpeed: TLabel;
    lablDeviceSimul: TLabel;
    FStandaloneServer: TIWStandAloneServer;
    FOperaExecutable: string;
    FNS6Executable: string;
    FMozillaExecutable: string;
    FIEExecutable: string;
    //
    procedure AboutOnClick(ASender: TObject);
    procedure actnAboutOnExecute(ASender: TObject);
    procedure actnClearSessionsOnExecute(ASender: TObject);
    procedure actnCopyURLToClipboardOnExecute(ASender: TObject);
    procedure actnExitOnExecute(ASender: TObject);
    procedure actnExecuteDefaultExecute(ASender: TObject);
    procedure actnExecuteOpera(ASender: TObject);
    procedure actnExecuteMozilla(ASender: TObject);
    procedure actnExecuteNS6(ASender: TObject);
    procedure actnExecuteIE(ASender: TObject);
    procedure actnShowDebugInformationOnExecute(ASender: TObject);
    procedure actnTestOnExecute(ASender: TObject);
    procedure actnSetSSL(ASender: TObject);
    procedure SetActiveSessions;
    procedure SetConnectionSpeed(AValue: Integer);
    procedure OnCreateSession(ASender: TObject);
    procedure OnDestroySession(ASender: TObject);
    procedure ActionList1Update(Action: TBasicAction; var Handled: Boolean);
    procedure OnDebugLog(ASender: TObject; ALog: String);
    procedure InitializeToolbar(AToolBar: TToolBar);
    procedure UpdateBrowserSelectionAction;
    procedure DoOnDeviceChange(Sender: TObject);
    procedure DoOnSpeedChange(Sender: TObject);
    procedure DoOnCustomSpeedChange(Sender: TObject);
    procedure DoOnParametersChange(Sender: TObject);
  protected
    FDeviceCombo : TComboBox;
    FSpeedCombo : TComboBox;
    FCustomSpeedEdit : TEdit;
    FCustomSpeedLabel : TLabel;
    FParameters : TEdit;
    FBitsPerSec : integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Log(const AMsg: string);
  end;

var
  FormIWMain: TformIWMain;

implementation

{$IFDEF CLR}
  {$R IWMainToolbarImages.resx}
  {$R IWMainIcons.resx}
  {$R IWMainAboutSplash.resx}
{$ELSE}
  {$R IWMain.res}
{$ENDIF}

uses
  {$IFDEF Linux}QMenus,{$ELSE}Menus,{$ENDIF}
  {$IFDEF Linux}QImgList,{$ELSE}ImgList,{$ENDIF}
  {$IFDEF Linux}
    IWJPeg,
  {$ELSE}
    {$IFDEF CLR}
    IWNETJpeg,
    {$ELSE}
    JPeg,
    {$ENDIF}
  {$ENDIF}
  {$IFDEF Linux}Libc, Qt,{$ELSE}ShellAPI, Registry,{$ENDIF}
  IWApplication, IWGlobal, IWServerControllerBase, IWResourceStrings,
  IWUtils,
  InSync, INIFiles;

Type
  TIWMainUpdateSessions = class(TIdNotifyMethod)
  public
    class procedure UpdateSessions; 
  end;

// This is here to avoid having to reference Menus/QMenus when ShortCut is called as it conflicts
// in the with statements with the ShortCut property
function MakeShortCut(AKey: Word; AShift: TShiftState): TShortCut;
begin
  Result := ShortCut(AKey, AShift);
end;

constructor TformIWMain.Create(AOwner: TComponent);
var
  LIcon: TIcon;
  LMainMenu: TMainMenu;
  LMenuItem: TMenuItem;
  LMenuItem1: TMenuItem;
  LMenuItemHead: TMenuItem;
  LPanel: TPanel;
  LStream: TResourceStream;
  LToolBar: TToolbar;

  LDeviceLabel : TLabel;
  LSpeedLabel : TLabel;
  LParametersLabel : TLabel;

  {$IFDEF CLR}
  LRM : ResourceManager;
  LBitmap : TImage;
  {$ENDIF}

  {$IFDEF Linux}
    function ResourceLoad(AImageList: TCustomImageList; AResName: String;
      AMaskColor: TColor): Boolean;
    Var
      LBMP: TBitmap;
    begin
      LBMP := TBitmap.Create; try
        LBMP.LoadFromResourceName(GetModuleHandle(nil), AResName);
        AImageList.AddMasked(LBMP, AMaskColor);
      finally FreeAndNil(LBMP); end;
      result := true;
    end;
  {$ENDIF}
begin
  // DO NOT call inherited!
  CreateNew(AOwner);
  //
  Width := 500;
  Height := 300;
  if Length(Application.Title) > 0 then begin
    Caption := Application.Title + ' Server';
  end else begin
    Caption := 'Intraweb Server';
  end;
  Color := clBtnFace;
  Position := poDefaultPosOnly;
  // Set Icon
  {$IFDEF CLR}
  Application.Icon.LoadFromResourceName('ICO_INTRAWEB', 'IWMainIcons', System.Reflection.Assembly.GetExecutingAssembly);
  {$ELSE}
  LIcon := TIcon.Create;
  try
    LStream := TResourceStream.Create(GetModuleHandle(nil), 'IW_MAIN_ICON', RT_RCDATA); 
    try
      LIcon.LoadFromStream(LStream);
    finally
      FreeAndNil(LStream);
    end;
    Application.Icon.Assign(LIcon);
  finally
    FreeAndNil(LIcon);
  end;
  {$ENDIF}
  // Construct Memo in order to be used for logging
  FMemoLog := TMemo.Create(Self);
  with FMemoLog do begin
    Parent := Self;
    Left := 169;
    Top := 41;
    Width := 232;
    Height := 223;
    Align := alClient;
    ReadOnly := True;
    ScrollBars := ssVertical;
    TabOrder := 0;
    {$IFDEF Linux}
    Font.Size := 9;
    {$ENDIF}
  end;
  // Constructor Standalone Server component
  try
    FStandaloneServer := TIWStandAloneServer.Create(self, OnDebugLog);
    // Construct Actions
    FActionList := TActionList.Create(Self);
    FActionList.Images := TImageList.Create(Self);
    FActionList.OnUpdate := ActionList1Update;

    with FActionList.Images do begin
      // TODO: Does not work in Linux and will be removed from here after
      // Porting to Linux
      {$IFNDEF Linux}
      {$IFDEF CLR}
      try
        TransparentColor := True;
        TransparentColorValue := clDkGray;

        LBitmap := TImage.Create(nil);
        try
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_HTTP', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_HTTPS', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_PARAMS', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_DEBUG', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_COPY', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_CLEAN', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_ABOUT', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_SIMULATEHTML32', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_SETSPEED', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_IE', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_NETSCAPE', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_MOZILLA', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_OPERA', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_SIMULATEWML', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
          LBitmap.Picture.Bitmap.LoadFromResourceName('BTN_SIMULATEXHTMLMP', 'IWMainToolbarImages', System.Reflection.Assembly.GetExecutingAssembly);
          Add(LBitmap.Picture.Bitmap, nil);
        except
          raise Exception.Create(RSToolBarError);
        end
      finally
        FreeAndNil(LBitmap);
      end;
      {$ELSE}
      if (not ResourceLoad(rtBitmap, 'IW_MAIN_HTTP', clNone)) // 0
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_HTTPS', clNone)) // 1
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_PARAMS', clNone)) // 2
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_DEBUG', clNone)) // 3
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_COPY', clNone)) // 4
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_CLEAN', clNone)) // 5
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_ABOUT', clNone)) // 6
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_32', clNone)) // 7
       or (not ResourceLoad(rtBitmap, 'IW_SPEED_LIMIT', clNone)) // 8
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_IE', clNone)) // 9
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_NS6', clNone)) // 10
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_MOZILLA', clNone)) // 11
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_OPERA', clNone)) // 12
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_WML', clNone))  // 13
       or (not ResourceLoad(rtBitmap, 'IW_MAIN_XHTMLMP', clNone)) // 14
       then begin
        raise Exception.Create(RSToolBarError);
      end;
      {$ENDIF}
      {$ELSE}
      if (not ResourceLoad(FActionList.Images, 'IW_MAIN_HTTP', clNone)) // 0
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_HTTPS', clNone)) // 1
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_PARAMS', clNone)) // 2
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_DEBUG', clNone)) // 3
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_COPY', clNone)) // 4
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_CLEAN', clNone)) // 5
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_ABOUT', clNone)) // 6
       or (not ResourceLoad(FActionList.Images, 'IW_MAIN_32', clNone)) // 7
       or (not ResourceLoad(FActionList.Images, 'IW_SPEED_LIMIT', clNone)) // 8
       then begin
        raise Exception.Create(RSToolBarError);
      end;
      {$ENDIF}
    end;
    // Clear Sessions
    FActionFile_ClearSessions := TAction.Create(Self);
    with FActionFile_ClearSessions do begin
      ActionList := FActionList;
      Caption := RSClearSessionsCaption;
      Hint := RSClearSessionsHint;
      ShortCut := MakeShortCut(Ord('K'), [ssCtrl]);
      ImageIndex := 5;
      OnExecute := actnClearSessionsOnExecute;
    end;
    // Copy URL to Clipboard
    FActionFile_CopyURLToClipboard := TAction.Create(Self);
    with FActionFile_CopyURLToClipboard do begin
      ActionList := FActionList;
      Caption := RSCopyClipboardCaption;
      Hint := RSCopyClipboardHint;
      ShortCut := MakeShortCut(Ord('C'), [ssCtrl]);
      ImageIndex := 4;
      OnExecute := actnCopyURLToClipboardOnExecute;
    end;
    // Exit
    FActionFile_Exit := TAction.Create(Self);
    with FActionFile_Exit do begin
      ActionList := FActionList;
      Caption := RSExitCaption;
      Hint := RSExitHint;
      OnExecute := actnExitOnExecute;
    end;
    // Execute
    FActionRun_Execute := TAction.Create(Self);
    with FActionRun_Execute do begin
      ActionList := FActionList;
      Caption := RSExecuteCaption;
      Hint := RSExecuteHint;
      ImageIndex := 0;
      ShortCut := MakeShortCut({$IFDEF Linux}Key_F9{$ELSE}VK_F9{$ENDIF}, []);
      OnExecute := actnTestOnExecute;
    end;

    FActionRun_ExecuteDefault := TAction.Create(Self);
    with FActionRun_ExecuteDefault do begin
      ActionList := FActionList;
      Caption := RSExecuteDefaultCaption;
      Hint := RSExecuteDefaultHint;
      ImageIndex := 0;
      OnExecute := actnExecuteDefaultExecute;
    end;
    FActionRun_ExecuteIE := TAction.Create(Self);
    with FActionRun_ExecuteIE do begin
      ActionList := FActionList;
      Caption := RSExecuteIECaption;
      Hint := RSExecuteIEHint;
      ImageIndex := 9;
      ShortCut := MakeShortCut(Ord('E'), [ssCtrl]);
      Enabled := FStandaloneServer.TestForIE;
      OnExecute := actnExecuteIE;
    end;
    FActionRun_ExecuteMozilla := TAction.Create(Self);
    with FActionRun_ExecuteMozilla do begin
      ActionList := FActionList;
      Caption := RSExecuteMozillaCaption;
      Hint := RSExecuteMozillaHint;
      ImageIndex := 11;
      Enabled := FStandaloneServer.TestForMozilla;
      ShortCut := MakeShortCut(Ord('M'), [ssCtrl]);
      OnExecute := actnExecuteMozilla;
    end;
    FActionRun_ExecuteNS6 := TAction.Create(Self);
    with FActionRun_ExecuteNS6 do begin
      ActionList := FActionList;
      Caption := RSExecuteNS6Caption;
      Hint := RSExecuteNS6Hint;
      ImageIndex := 10;
      Enabled := FStandaloneServer.TestForNS6;
      ShortCut := MakeShortCut(Ord('N'), [ssCtrl]);
      OnExecute := actnExecuteNS6;
    end;
    FActionRun_ExecuteOpera := TAction.Create(Self);
    with FActionRun_ExecuteOpera do begin
      ActionList := FActionList;
      Caption := RSExecuteOperaCaption;
      Hint := RSExecuteOperaHint;
      ImageIndex := 12;
      ShortCut := MakeShortCut(Ord('O'), [ssCtrl]);
      Enabled := FStandaloneServer.TestForOpera;
      OnExecute := actnExecuteOpera;
    end;

    // Options
    FActionOptions_ShowDebugInformation := TAction.Create(Self);
    with FActionOptions_ShowDebugInformation do begin
      ActionList := FActionList;
      Caption := RSShowDebugInfoCaption;
      Hint := RSShowDebugInfoHint;
      ImageIndex := 3;
      OnExecute := actnShowDebugInformationOnExecute;
    end;
    FActionOptions_ExecuteSSL := TAction.Create(Self);
    with FActionOptions_ExecuteSSL do begin
      ActionList := FActionList;
      Caption := RSUseSSLCaption;
      Hint := RSUseSSLHint;
      ImageIndex := 1;
      ShortCut := MakeShortCut({$IFDEF Linux}Key_F9{$ELSE}VK_F9{$ENDIF}, [ssCtrl]);
      OnExecute := actnSetSSL;
      {$IFDEF CLR}
      Visible := False;
      {$ELSE}
      Enabled := GServerController.SSLOptions.Port <> 0;
      {$ENDIF}
    end;
    // Help
    FActionHelp_About := TAction.Create(Self);
    with FActionHelp_About do begin
      ActionList := FActionList;
      Caption := RSAboutCaption;
      Hint := RSAboutHint;
      ImageIndex := 6;
      OnExecute := actnAboutOnExecute;
    end;

    {$IFDEF Linux}
    Font.Size := 9;
    {$ENDIF}

    // Construct Menu
    LMainMenu := TMainMenu.Create(Self);
    LMainMenu.Images := FActionList.Images;
    Menu := LMainMenu;
    //
    LMenuItemHead := TMenuItem.Create(Self);
    LMainMenu.Items.Add(LMenuItemHead);
    with LMenuItemHead do begin
      Caption := RSFileMenuCaption;
      // Clear Session
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionFile_ClearSessions;
      end;
      // Copy URL to Clipboard
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionFile_CopyURLToClipboard;
      end;
      // Seperator
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Caption := '-';
      end;
      // Exit
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionFile_Exit;
      end;
    end;
    // Run
    LMenuItemHead := TMenuItem.Create(Self);
    LMainMenu.Items.Add(LMenuItemHead);
    with LMenuItemHead do begin
      Caption := RSRunMenuCaption;
      // Execute
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionRun_Execute;
      end;
      // Seperator
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Caption := '-';
      end;
      // Select browser
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Caption := RSSelectBrowserCaption;
      end;

      // Execute default browser
      LMenuItem1 := TMenuItem.Create(Self);
      LMenuItem.Add(LMenuItem1);
      with LMenuItem1 do begin
        Action := FActionRun_ExecuteDefault;
      end;
      // Execute via IE
      LMenuItem1 := TMenuItem.Create(Self);
      LMenuItem.Add(LMenuItem1);
      with LMenuItem1 do begin
        Action := FActionRun_ExecuteIE;
      end;
      // Execute via Mozilla
      LMenuItem1 := TMenuItem.Create(Self);
      LMenuItem.Add(LMenuItem1);
      with LMenuItem1 do begin
        Action := FActionRun_ExecuteMozilla;
      end;
      // Execute via Netscape 6
      LMenuItem1 := TMenuItem.Create(Self);
      LMenuItem.Add(LMenuItem1);
      with LMenuItem1 do begin
        Action := FActionRun_ExecuteNS6;
      end;
      // Execute via Opera
      LMenuItem1 := TMenuItem.Create(Self);
      LMenuItem.Add(LMenuItem1);
      with LMenuItem1 do begin
        Action := FActionRun_ExecuteOpera;
      end;

      // Seperator
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Caption := '-';
      end;
    end;
    // Options
    LMenuItemHead := TMenuItem.Create(Self);
    LMainMenu.Items.Add(LMenuItemHead);
    with LMenuItemHead do begin
      Caption := RSOptionsMenuCaption;
      // Execute SSL
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionOptions_ExecuteSSL;
      end;
      // Show Debug Information
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionOptions_ShowDebugInformation;
      end;
    end;
    // Help
    LMenuItemHead := TMenuItem.Create(Self);
    LMainMenu.Items.Add(LMenuItemHead);
    with LMenuItemHead do begin
      Caption := RSHelpCaption;
      LMenuItem := TMenuItem.Create(Self);
      LMenuItemHead.Add(LMenuItem);
      with LMenuItem do begin
        Action := FActionHelp_About;
      end;
    end;
    LPanel := TPanel.Create(Self);
    with LPanel do begin
      ParentFont := true;
      Parent := Self;
      Left := 0;
      Top := 41;
      Width := 169;
      Height := 223;
      Align := alLeft;
      TabOrder := 1;
      lablActiveSessions := TLabel.Create(Self);
      with lablActiveSessions do begin
        Parent := LPanel;
        AutoSize := True;
        Left := 8;
        Top := 10;
      end;
      SetActiveSessions;
    end;
    FStatusBar := TStatusBar.Create(Self);
    with FStatusBar do begin
      ParentFont := true;
      Parent := Self;
    end;
    // Construct Toolbar
    LToolbar := TToolbar.Create(Self);
    with LToolbar do begin
      Name := RSToolBarCaption;
      Parent := Self;
      Images := FActionList.Images;
      ShowHint := True;
      InitializeToolbar(LToolBar);
      ParentFont := true;
    end;

    // Device simulation
    // Device label
    LDeviceLabel := TLabel.Create(Self);
    with LDeviceLabel do
    begin
      Name := 'LDeviceLabel';
      Parent := LPanel;
      Left := 8;
      Top := 30;
      Caption := 'Device simulation';
      ParentFont := true;
    end;

    // Device combo
    FDeviceCombo := TComboBox.Create(Self);
    with FDeviceCombo do
    begin
      Name := 'FDeviceCombo';
      Parent := LPanel;
      Left := 8;
      Top := 45;
      Items.Add('None');
      Items.Add('HTML 3.2 (PDA Mode)');
      Items.Add('WML (WAP Mode)');
      Items.Add('XHTML MP (WAP Mode)');
      ItemIndex := 0;
      Style := csDropDownList;
      OnChange := DoOnDeviceChange;
      Width := 150;
      ParentFont := true;
    end;

    // Simulated connection speed
    // Speed label
    LSpeedLabel := TLabel.Create(Self);
    with LSpeedLabel do
    begin
      Name := 'LSpeedLabel';
      Parent := LPanel;
      Left := 8;
      Top := 70;
      Caption := 'Simulated connection speed';
      ParentFont := true;
    end;

    // Speed combo
    FSpeedCombo := TComboBox.Create(Self);
    with FSpeedCombo do
    begin
      Name := 'FSpeedCombo';
      Parent := LPanel;
      Left := 8;
      Top := 85;
      Items.Add('No limit');
      Items.Add('Dial-Up 28.8kbps');
      Items.Add('Dial-Up 33.6kbps');
      Items.Add('ISDN 64kbps');
      Items.Add('ISDN 128kbps');
      Items.Add('DSL/Cable 384kbps');
      Items.Add('Ethernet 10mbps');
      Items.Add('Custom...');
      ItemIndex := 0;
      Style := csDropDownList;
      OnChange := DoOnSpeedChange;
      Width := 150;
      ParentFont := true;
    end;

    // Custom speed label
    FCustomSpeedLabel := TLabel.Create(Self);
    with FCustomSpeedLabel do
    begin
      Parent := LPanel;
      Left := 8;
      Top := 110;
      Caption := 'Custom speed (bps)';
      Visible := False;
      ParentFont := true;
    end;

    // Custom speed edit
    FCustomSpeedEdit := TEdit.Create(Self);
    with FCustomSpeedEdit do
    begin
      Parent := LPanel;
      Text := '0';
      Left := 8;
      Top := 125;
      OnChange := DoOnCustomSpeedChange;
      Visible := False;
      Width := 150;
      ParentFont := true;
    end;

    // Parameters
    // Parameters label
    LParametersLabel := TLabel.Create(Self);
    with LParametersLabel do
    begin
      Parent := LPanel;
      Left := 8;
      Top := 150;
      Caption := 'Application parameters';
      ParentFont := true;
    end;

    // Parameters box
    FParameters := TEdit.Create(Self);
    with FParameters do
    begin
      Text := '0';
      Parent := LPanel;
      Left := 8;
      Top := 165;
      OnChange := DoOnParametersChange;
      Text := '';
      Width := 150;
      ParentFont := true;
    end;

    //
    FStandaloneServer.OnNewSession := OnCreateSession;
    FStandaloneServer.OnCloseSession := OnDestroySession;

    // Read saved params
    with TINIFile.Create(ChangeFileExt(ParamStr(0), '.iwb')) do try
      FStandaloneServer.Params := ReadString('Main', 'Parameters', '');
      FStandaloneServer.DebugLog := ReadBool('Main', 'DebugInfo', false);
      FStandaloneServer.RunSSL := ReadBool('Main', 'SSL', false);
      FStandaloneServer.HTML32 := ReadBool('Main', 'HTML32', false);
      FSTandAloneServer.WML := REadBool('Main', 'WML', False);
      FSTandAloneServer.XHTMLMP := REadBool('Main', 'XHTMLMP', False);
      FStandaloneServer.RunIn := TIWWebBrowser(ReadInteger('Main', 'RunIn', Integer(wbDefault)));
      FBitsPerSec := ReadInteger('HTTP', 'SpeedLimit', 0);
      SetConnectionSpeed(FBitsPerSec);

      if FStandaloneServer.HTML32 then
        FDeviceCombo.ItemIndex := 1;
      if FStandaloneServer.WML then
        FDeviceCombo.ItemIndex := 2;
      if FStandaloneServer.XHTMLMP then
        FDeviceCombo.ItemIndex := 3;
      FParameters.Text := FStandaloneServer.Params;

      case FBitsPerSec of
              0   : FSpeedCombo.ItemIndex := 0;
          28800   : FSpeedCombo.ItemIndex := 1;
          33600   : FSpeedCombo.ItemIndex := 2;
          64000   : FSpeedCombo.ItemIndex := 3;
         128000   : FSpeedCombo.ItemIndex := 4;
         384000   : FSpeedCombo.ItemIndex := 5;
       10000000   : FSpeedCombo.ItemIndex := 6;
      else
      begin
        FSpeedCombo.ItemIndex := 7;
        FCustomSpeedEdit.Text := IntToStr(FBitsPerSec);
      end
      end;

      DoOnSpeedChange(Self);

    finally Free; end;
    UpdateBrowserSelectionAction;
  except
    on E: Exception do begin
      MessageDlg(E.Message, mtError, [mbOK], -1);
      Application.Terminate;
    end;
  end;
end;

procedure TformIWMain.actnClearSessionsOnExecute(ASender: TObject);
begin
  FStandaloneServer.ClearSessions;
end;

procedure TformIWMain.SetActiveSessions;
begin
  lablActiveSessions.Caption := RSLogActiveSessions + IntToStr(FStandaloneServer.ActiveSessions);
end;

procedure TformIWMain.actnExitOnExecute(ASender: TObject);
begin
  Close;
end;

procedure TformIWMain.actnAboutOnExecute(ASender: TObject);
var
  LLabel: TLabel;
  LVerLabel: TLabel;
  LImage: TImage;
  LForm: TForm;
  LStream: TStream;
begin
  LForm := TForm.Create(Application); try
    with LForm do begin
      Position := poScreenCenter;
      {$IFDEF Linux}
      BorderStyle := fbsNone;
      {$ELSE}
      BorderStyle := bsNone;
      {$ENDIF}
      VertScrollBar.Visible := false;
      HorzScrollBar.Visible := false;
      Caption := 'About IntraWeb';
    end;
    LImage := TImage.Create(LForm);
    with LImage do begin
      Parent := LForm;
      Name := 'LImage';
      Left := 0;
      Top := 0;
      AutoSize := True;
      {$IFDEF CLR}
      LImage.Picture.Bitmap.LoadFromResourceName('IW_Splash', 'IWMainAboutSplash', System.Reflection.Assembly.GetExecutingAssembly);
      {$ELSE}
      LStream := TResourceStream.Create(GetModuleHandle(nil), 'IW_SPLASH_ABOUT', RT_RCDATA); try
        // {$IFDEF Linux}
        // LImage.Picture.(LStream);
        // {$ELSE}
        LImage.Picture.Graphic := TJPEGImage.Create;
        LImage.Picture.Graphic.LoadFromStream(LStream);
        // {$ENDIF}
      finally FreeAndNil(LStream); end;
      {$ENDIF}
      Align := alClient;
      OnClick := AboutOnClick;
    end;

    LLabel := TLabel.Create(LForm);
    with LLabel do begin
      Parent := LForm;
      AutoSize := True;
      Name := 'LLabel';
      Left := 105;
      Top := 175;
      Caption := 'Version ' + GVersion;
      Transparent := True;
      OnClick := AboutOnClick;
    end;
    LVerLabel := TLabel.Create(LForm);
    with LVerLabel do begin
      Parent := LForm;
      AutoSize := true;
      Name := 'LVerLable';
      Font.Size := 70;
      Font.Color := $000CC999;
      Font.Style := [fsBold];
      Font.Name := 'Times New Roman';
      Caption := Copy(GVersion, 1, 1);
      OnClick := AboutOnClick;
      Left := 10;
      Top := 160;
      Transparent := True;
    end;
    LForm.Width := LImage.Width;
    LForm.Height := LImage.Height;

    LForm.ShowModal;
  finally FreeAndNil(LForm) end;
end;

procedure TformIWMain.actnCopyURLToClipboardOnExecute(ASender: TObject);
begin
  FStandaloneServer.CopyStartURLToClipboard;
end;

procedure TFormIWMain.AboutOnClick(ASender: TObject);
begin
  TForm(TImage(ASender).Owner).ModalResult := mrOk;
end;

procedure TFormIWMain.Log(const AMsg: string);
begin
  FMemoLog.Lines.Add(AMsg);
end;

procedure TFormIWMain.OnCreateSession(ASender: TObject);
begin
  TIWMainUpdateSessions.UpdateSessions;
end;

procedure TFormIWMain.OnDestroySession(ASender: TObject);
begin
  TIWMainUpdateSessions.UpdateSessions;
end;

destructor TFormIWMain.Destroy;
begin
  if Assigned(FStandaloneServer) then begin
    with TINIFile.Create(ChangeFileExt(ParamStr(0), '.iwb')) do try
      WriteString('Main', 'Parameters', FStandAloneServer.Params);
      WriteBool('Main', 'SSL', FStandAloneServer.RunSSL);
      WriteBool('Main', 'DebugInfo', FStandAloneServer.DebugLog);
      WriteBool('Main', 'HTML32', FStandAloneServer.HTML32);
      WriteBool('Main', 'WML', FSTandaLoneServer.WML);
      WriteBool('Main', 'XHTMLMP', FSTandaLoneServer.XHTMLMP);
      WriteInteger('Main', 'RunIn', Integer(FStandAloneServer.RunIn));
      WriteInteger('HTTP', 'SpeedLimit', GHTTPServer.BitsPerSec);
      UpdateFile;
    finally Free; end;
  end;
  {GSessions.OnCreateSession := nil;
  GSessions.OnDestroySession := nil;}
  inherited Destroy;
end;

procedure TFormIWMain.InitializeToolbar(AToolBar: TToolBar);

  procedure Seperator;
  begin
    with TToolButton.Create(AToolBar.Owner) do begin
      Parent := AToolBar;
      Style := tbsSeparator;
      Width := 4;
    end;
  end;

begin
{$IFNDEF Linux}
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionHelp_About;
    Parent := AToolBar;
  end;
  Seperator;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionOptions_ExecuteSSL;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionOptions_ShowDebugInformation;
    Parent := AToolBar;
  end;
  Seperator;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionFile_ClearSessions;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionFile_CopyURLToClipboard;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := Self.FActionRun_Execute;
    Parent := AToolBar;
  end;
{$ENDIF}
{$IFDEF Linux}
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionRun_Execute;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionOptions_ExecuteSSL;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionFile_CopyURLToClipboard;
    Parent := AToolBar;
  end;
  Seperator;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionFile_ClearSessions;
    Parent := AToolBar;
  end;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionOptions_ShowDebugInformation;
    Parent := AToolBar;
  end;
  Seperator;
  with TToolButton.Create(AToolBar.Owner) do begin
    Action := formIWMain.FActionHelp_About;
    Parent := AToolBar;
  end;
{$ENDIF}
end;

procedure TFormIWMain.actnShowDebugInformationOnExecute(ASender: TObject);
begin
  FStandaloneServer.DebugLog := not FStandaloneServer.DebugLog;
end;

procedure TFormIWMain.actnExecuteDefaultExecute(ASender: TObject);
begin
  FStandaloneServer.RunIn := wbDefault;
  UpdateBrowserSelectionAction;
end;

procedure TFormIWMain.actnExecuteIE(ASender: TObject);
begin
  FStandaloneServer.RunIn := wbIE;
  UpdateBrowserSelectionAction;
end;

procedure TFormIWMain.actnExecuteMozilla(ASender: TObject);
begin
  FStandaloneServer.RunIn := wbMozilla;
  UpdateBrowserSelectionAction;
end;

procedure TFormIWMain.actnExecuteNS6(ASender: TObject);
begin
  FStandaloneServer.RunIn := wbNS6;
  UpdateBrowserSelectionAction;
end;

procedure TFormIWMain.actnExecuteOpera(ASender: TObject);
begin
  FStandaloneServer.RunIn := wbOpera;
  UpdateBrowserSelectionAction;
end;

procedure TFormIWMain.ActionList1Update(Action: TBasicAction;
  var Handled: Boolean);
begin
  FActionOptions_ShowDebugInformation.Checked := FStandaloneServer.DebugLog;
  FActionOptions_ExecuteSSL.Checked := FStandaloneServer.RunSSL;
  Handled := True;
end;

procedure TFormIWMain.actnTestOnExecute(ASender: TObject);
begin
  FStandaloneServer.Run;
end;

procedure TFormIWMain.OnDebugLog(ASender: TObject; ALog: String);
begin
  Log(ALog);
end;

{ TIWMainUpdateSessions }

class procedure TIWMainUpdateSessions.UpdateSessions;
begin
  with Create(formIWMain.SetActiveSessions) do begin
    Notify;
  end;
end;

procedure TFormIWMain.UpdateBrowserSelectionAction;
Var
  LSelected: TAction;
begin
  LSelected := FActionRun_ExecuteDefault;
  FActionRun_ExecuteDefault.Checked := false;
  FActionRun_ExecuteIE.Checked := false;
  FActionRun_ExecuteMozilla.Checked := false;
  FActionRun_ExecuteNS6.Checked := false;
  FActionRun_ExecuteOpera.Checked := false;
  case FStandaloneServer.RunIn of
    wbDefault: LSelected := FActionRun_ExecuteDefault;
    wbIE: LSelected := FActionRun_ExecuteIE;
    wbNS6: LSelected := FActionRun_ExecuteNS6;
    wbMozilla: LSelected := FActionRun_ExecuteMozilla;
    wbOpera: LSelected := FActionRun_ExecuteOpera;
  end;
  LSelected.Checked := true;
  FActionRun_Execute.ImageIndex := LSelected.ImageIndex;
end;

procedure TFormIWMain.actnSetSSL(ASender: TObject);
begin
  FStandaloneServer.RunSSL := not FActionOptions_ExecuteSSL.Checked;
end;


procedure TFormIWMain.SetConnectionSpeed(AValue: Integer);
begin
  GHTTPServer.BitsPerSec := FBitsPerSec;
end;

procedure TFormIWMain.DoOnDeviceChange(Sender: TObject);
begin
  FStandaloneServer.HTML32 := FDeviceCombo.ItemIndex = 1;
  FStandaloneServer.WML := FDeviceCombo.ItemIndex = 2;
  FStandaloneServer.XHTMLMP := FDeviceCombo.ItemIndex = 3;
end;

procedure TFormIWMain.DoOnSpeedChange(Sender: TObject);
begin
  FCustomSpeedLabel.Visible := FSpeedCombo.ItemIndex = 7;
  FCustomSpeedEdit.Visible := FSpeedCombo.ItemIndex = 7;

  case FSpeedCombo.ItemIndex of
    0 : FBitsPerSec := 0;
    1 : FBitsPerSec := 28800;
    2 : FBitsPerSec := 33600;
    3 : FBitsPerSec := 64000;
    4 : FBitsPerSec := 128000;
    5 : FBitsPerSec := 384000;
    6 : FBitsPerSec := 10000000;
    7 : FBitsPerSec := StrToIntDef(FCustomSpeedEdit.Text, 0);
  end;
  SetConnectionSpeed(FBitsPerSec);
end;

procedure TFormIWMain.DoOnCustomSpeedChange(Sender: TObject);
begin
  FBitsPerSec := StrToIntDef(FCustomSpeedEdit.Text, 0);
  SetConnectionSpeed(FBitsPerSec);
end;

procedure TFormIWMain.DoOnParametersChange(Sender: TObject);
begin
  FStandaloneServer.Params := FParameters.Text;
end;

end.


