unit IWRegion;

interface

uses
  {$IFDEF VSNET}
    System.ComponentModel,
    System.ComponentModel.Design,
    System.Drawing,

    IWNetBaseControl,
    IWNetBaseContainer,
    IWNetComponent,
    IWNetClasses,
  {$ELSE}
    {$IFDEF Linux}
      IWCLXBaseControl,
    {$ELSE}
      IWVCLBaseControl,
      IWVCLBaseContainer,
    {$ENDIF}
  {$ENDIF}

  {$IFDEF VSNET}
  Graphics,
  {$ELSE}
  Classes,
    {$IFDEF Linux}
    IWCLXComponent, IWCLXClasses,
    {$ELSE}
    IWVCLComponent, IWVCLClasses,
    {$ENDIF}
  {$IFDEF Linux}QControls,{$ELSE}Controls,{$ENDIF}
  {$IFDEF Linux}QForms,{$ELSE}Forms,{$ENDIF}
  {$IFDEF Linux}QGraphics,{$ELSE}Graphics,{$ENDIF}
  {$IFDEF Linux}Qt, Types,{$ELSE}Windows, Messages,{$ENDIF}
  {$ENDIF}
  IWContainer, IWControl, IWHTMLTag, IWBaseContainerLayout,
  IWColor, IWBaseInterfaces, IWRenderContext, IWContainerLayout, IWBaseHTMLInterfaces,
  IWBaseHTMLControl, IWGridCommon, IWHTML40Interfaces, IWScriptEvents, IWTypes,
  IWContainerBorderOptions, IWApplication, IWFont, IWBaseControl,
  IWMarkupLanguageTag, IWHTMLContainer, IWBaseRenderContext;

type
  {$IFDEF VSNET}
  [
   ToolboxItemFilter('IWAppForm.TIWAppForm'),
   TIWToolPalette('Intraweb Controls')
  ]
  {$ENDIF}
  TIWCustomRegion = class(TIWHTMLContainer, IIWHTML40Container, IIWBaseComponent, IIWBaseHTMLComponent, IIWBaseControl, IIWBaseHTMLControl, IIWHTML40Control, IIWHTML40Component, IIWTabOrder)
  private
    FBorderOptions: TIWContainerBorderOptions;
    FIWControlImplementation: TIWHTMLControlImplementation;
    FIWBaseControlImplementation: TIWBaseControlImplementation;
    FDoRefreshControl: Boolean;
    FOnCreate: TNotifyEvent;
    FUseFrame: Boolean;
    FRegionDIV: TIWHTMLTag;
    FWebApplication: TIWApplication;

    procedure SetBorderOptions(const Value: TIWContainerBorderOptions);
    procedure OnBorderChange(ASender: TObject);
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  protected
    {$IFDEF VSNET}
    procedure AdjustClientRect(var Rect: Rectangle); override;
    function GetClientRect: Rectangle; override;
    {$ELSE}
    procedure AdjustClientRect(var Rect: TRect); override;
    function GetClientRect: TRect; override;
    {$ENDIF}

    function InitPaintHandler: TIWPaintHandler; override;

    // IIWContainer methods
    function InitContainerContext(AWebApplication: TIWApplication): TIWContainerContext; override;

    // IIWControl methods
    function get_HTMLLeft: Integer;
    function get_HTMLTop: Integer;

    function RenderCSSClass(AComponentContext: TIWBaseHTMLComponentContext): string;
    function RenderStyle(AComponentContext: TIWBaseHTMLComponentContext): string;
    procedure RenderScripts(AComponentContext: TIWBaseHTMLComponentContext);

    function RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag; override;

    function WantsSubmitValidation: Boolean;

    procedure set_WebFont(AValue: TIWFont);
    function get_WebFont: TIWFont;

    function get_Canvas: TCanvas; override;

    function ParentContainer: IIWBaseContainer;
    procedure PaintTo(ACanvas: TCanvas);

    function SupportsSubmit: Boolean;
    function SupportsInput: Boolean;
    procedure AddFreeNotifier(AObject: TObject);
    procedure RemoveFreeNotifier(AObject: TObject);
    function ComponentContextClass: TIWBaseComponentContextClass; virtual;

    function MakeHTMLTag(ATag: TIWHTMLTag): string;
    procedure DoHTMLTag(ATag: TIWHTMLTag);
    procedure  DoCreate;
    //
    procedure InitDesignTime; override;
    procedure InitControl; override;

    {$IFDEF VSNET}
    procedure SetName(const AValue: String); override;
    {$ELSE}
    procedure SetName(const AValue: TComponentName); override;
    {$ENDIF}

    procedure RenderComponents(AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext); override;

    function NeedsFormTag: Boolean;

    function get_RenderSize: Boolean;
    procedure set_RenderSize(AValue: Boolean);

    function get_ScriptEvents: TIWScriptEvents;
    procedure set_ScriptEvents(Value: TIWScriptEvents);

    function get_Clip: Boolean;
    procedure set_Clip(AValue: Boolean);

    function HintEvents(ATag: TIWHTMLTag{; const AHint: string}): string;
    procedure HookEvents(APageContext: TIWPageContext40;
      AScriptEvents: TIWScriptEvents); {$IFDEF Linux} reintroduce; {$ENDIF} virtual;

    procedure SetActiveControl(AControl: IIWHTML40Control);

    function GenerateControlPositions: String;

    function CheckComponentForRender(AComponent: TPlatformComponent): Boolean; override;
    function DoPostRenderProcessing(ATag: TIWMarkupLanguageTag;
      AComponentContext: TIWBaseComponentContext; AComponent: TPlatformComponent): TIWMarkupLanguageTag; override;

    function CheckUpdateTree(AComponent: TPlatformComponent): Boolean; override;

    // IIWBaseComponent interface
    function get_ComponentName: String;

    function get_SupportsPartial: Boolean;
    procedure set_SupportsPartial(AValue: BOolean);

    function getTabOrder: TIWTabOrder; override;
    procedure setTabOrder(AValue: TIWTabOrder); override;

    property SupportsPartial: Boolean read get_SupportsPartial write set_SupportsPartial;

    property ControlImplementation: TIWHTMLControlImplementation read FIWControlImplementation; // implements IIWBaseComponent, IIWBaseControl, IIWControl;
  public
    {$IFNDEF VSNET}
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure Invalidate; override;
    {$ENDIF}
    procedure Loaded; override;

    function get_DoRefreshControl: Boolean;
    procedure set_DoRefreshControl(AValue: Boolean);

    function get_HTMLName: String;

    function get_UseFrame: Boolean;
    procedure set_UseFrame(AValue: Boolean);

    function get_HTMLWidth: Integer;
    function get_HTMLHeight: Integer;

    property DoRefreshControl : Boolean read get_DoRefreshControl write set_DoRefreshControl;
    property HTMLName: string read get_HTMLName;

    property HTMLWidth: Integer read get_HTMLWidth;
    property HTMLHeight: Integer read get_HTMLHeight;

    property UseFrame: Boolean read get_UseFrame write set_UseFrame;
    {$IFNDEF CLR}
    property WebColor;
    {$ENDIF}
  published
    function get_WebCursor: TIWCursor;
    procedure set_WebCursor(AValue: TIWCursor);

    function get_Visible: Boolean; override;
    procedure set_Visible(Value: Boolean); override;

    function get_ExtraTagParams: TIWStringList;
    procedure set_ExtraTagParams(const Value: TIWStringList);

    function get_OnHTMLTag: TIWOnHTMLTag;
    procedure set_OnHTMLTag(AValue: TIWOnHTMLTag);

    procedure set_LayoutMgr(Value: TIWContainerLayout);
    function get_LayoutMgr: TIWContainerLayout;

    function get_ZIndex: Integer;
    procedure set_ZIndex(AValue: Integer);

    {$IFNDEF CLR}
    procedure set_WebColor(Value: TIWColor); override;
    function get_WebColor: TIWColor; override;
    {$ENDIF}

    property Align;
    property Anchors;
    {$IFDEF VSNET}
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    {$ENDIF}
    property BorderOptions: TIWContainerBorderOptions read FBorderOptions write SetBorderOptions;
    property TabOrder;
    {$IFDEF CLR}
    property WebColor;
    {$ELSE}
    property Color;
    {$ENDIF}
    property ParentShowHint;
    property ShowHint;
    property Hint;

    {$IFDEF CLR}
    // TODO: Remove this when .NET Delphi compiler is fixed
    property Visible read get_Visible write set_Visible;
    {$ELSE}
    property Visible;
    {$ENDIF}

    {$IFDEF CLR}
    {$IFDEF VSNET}
    [Category('Appearance')]
    {$ENDIF}
    property WebCursor: TIWCursor read get_WebCursor write set_WebCursor;
    {$ELSE}
    property Cursor: TIWCursor read get_WebCursor write set_WebCursor;
    {$ENDIF}

    property ClipRegion: Boolean read get_Clip write set_Clip default true;
    property LayoutMgr: TIWContainerLayout read get_LayoutMgr write set_LayoutMgr;
    {$IFDEF VSNET}
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    {$ENDIF}
    property ExtraTagParams: TIWStringList read get_ExtraTagParams write set_ExtraTagParams;
    property ZIndex: Integer read get_ZIndex write set_ZIndex;
    property OnHTMLTag: TIWOnHTMLTag read get_OnHTMLTag write set_OnHTMLTag;
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
  end;

  {$IFDEF VSNET}
  [
    Designer('Atozed.Intraweb.Design.UserControlDesigner, Atozed.Intraweb.Design', typeof(IRootDesigner)),
    DesignerCategory('UserControl')
  ]
  TIWUserControl = class(TIWCustomRegion)
  private
    FOnCreate: TNotifyEvent;
    FSetSizeCount: integer;
  protected
    procedure setSize(Value: Size); override;
    {procedure RequestAlign; override;}
    function ContainerPrefix: string; override;
    procedure Loaded; override;
    procedure Paint; override;
    function InitPaintHandler: TIWPaintHandler; override;

    procedure InitControl; override;
  published
    property OnCreate: TNotifyEvent read FOnCreate write FOnCreate;
  end;
  {$ENDIF}

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWRegion.TIWRegion.bmp}
  TIWRegion = class;
  [
    ToolboxItem(true), ToolboxBitmap(typeof(TIWRegion), 'TIWRegion.bmp'),
    Designer('Atozed.Intraweb.Design.IWContainerDesinger, Atozed.Intraweb.Design', typeof(IDesigner)),
    DesignerCategory('Component')
  ]
  {$ENDIF}
  TIWRegion = class(TIWCustomRegion)
  protected
    function ContainerPrefix: string; override;
  end;

implementation

uses
  {$IFDEF VSNET}
  Classes,
  {$ENDIF}
  SysUtils,
  IWForm, IWLayoutMgrForm,
  SWSystem, IWBaseHTMLComponent;

procedure TIWCustomRegion.Loaded;
begin
  inherited Loaded;
  DoCreate;
end;

function TIWCustomRegion.CheckComponentForRender(AComponent: TPlatformComponent): Boolean;
begin
  if SupportsInterface(Parent, IIWHTML40Form) then begin
    result := HTML40FormInterface(Parent).PageContext.UpdateMode = umPartial;
  end else begin
    if SupportsInterface(Parent, IIWHTML40Container) then begin
      result := HTML40ContainerInterface(Parent).CheckComponentForRender(AComponent);
    end else begin
      {$IFNDEF VSNET}
      if Parent is TFrame then begin
        if SupportsInterface(Parent.Parent, IIWHTML40Container) then begin
          result := HTML40ContainerInterface(Parent.Parent).CheckComponentForRender(AComponent);
        end else begin
          result := false;
        end;
      end else {$ENDIF} begin
        result := false;
      end;
    end;
  end;
end;

function TIWCustomRegion.DoPostRenderProcessing(ATag: TIWMarkupLanguageTag;
  AComponentContext: TIWBaseComponentContext; AComponent: TPlatformComponent): TIWMarkupLanguageTag;
var
  LIWControl: IIWHTML40Control;
  LTag: TIWHTMLTag;
begin
  result := ATag;
  LTag := TIWHTMLTag(ATag);
  BaseHTMLControlInterface(AComponent).DoHTMLTag(LTag);
  if SupportsInterface(AComponent, IIWHTML40Control) then begin
    LIWControl := HTML40ControlInterface(AComponent);
    if LIWControl.UseFrame and TIWContainerLayout(ContainerContext.LayoutManager).AllowFrames then begin
      result := TIWContainerBorderOptions.CreateDefaultFrame(LTag, TIWPageCOntext40(AComponentContext.BasePageContext), LIWControl);
    end;

    AComponentContext.MarkupLanguageTag := Result;

    HTML40ComponentInterface(AComponent).RenderScripts(TIWBaseHTMLComponentContext(AComponentContext));
  end;
end;

function TIWCustomRegion.CheckUpdateTree(AComponent: TPlatformComponent): Boolean;
begin
  result := false;
  if SupportsInterface(AComponent, IIWHTML40Component) then begin
    result := HTML40ComponentInterface(AComponent).DoRefreshControl;
  end;
end;

procedure TIWCustomRegion.SetBorderOptions(const Value: TIWContainerBorderOptions);
begin
  FBorderOptions.Assign(Value);
  BorderOptions.Color := FBorderOptions.Color;
  BorderOptions.NumericWidth := FBorderOptions.NumericWidth;
  BorderOptions.Style := FBorderOptions.Style;
  BorderOptions.BorderWidth := FBorderOptions.BorderWidth;
  RequestAlign;
  Invalidate;
end;

procedure TIWCustomRegion.InitControl;
begin
  FIWControlImplementation := TIWHTMLControlImplementation.Create(Self);
  FIWBaseControlImplementation := TIWBaseControlImplementation.Create(Self);

  inherited;

  FBorderOptions := TIWContainerBorderOptions.Create;
  FBorderOptions.Color := fromTColor(clNone);
  FBorderOptions.Style := cbsSolid;
  FBorderOptions.BorderWidth := cbwNumeric;
  FBorderOptions.NumericWidth := 1;

  FBorderOptions.OnChange := OnBorderChange;

  {$IFNDEF VSNET}
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csSetCaption, csDoubleClicks];
  {$ENDIF}
  WebColor := fromTColor(clNone);
  Width := 60;
  Height := 60;
  set_ZIndex(-1);
  ClipRegion:= true;
  ShowHint := True;
end;

procedure TIWCustomRegion.InitDesignTime;
begin
  inherited;

  if IsDesignMode then begin
    {$IFNDEF VSNET}
    ControlState := [csCustomPaint];
    {$IFDEF Linux}
    VertScrollBar.Visible := false;
    HorzScrollBar.Visible := false;
    {$ENDIF}
    {$ENDIF}
  end;
  ClipRegion:= true;
end;

function TIWCustomRegion.NeedsFormTag: Boolean;
begin
  result := false;
end;

function TIWCustomRegion.RenderHTML(AContext: TIWBaseHTMLComponentContext): TIWHTMLTag;
begin
  FWebApplication := AContext.WebApplication;

  {$IFDEF VSNET}
  BorderOptions.InitalizeForRendering(WebColor, false, false);
  {$ELSE}
  BorderOptions.InitalizeForRendering(WebColor, HorzScrollBar.Visible, VertScrollBar.Visible);
  {$ENDIF}
  FregionDiv := BorderOptions.CreateFrame(nil, TIWPageContext40(AContext.PageContext), Self);
  result := FregionDiv;
end;

(*procedure TIWRegion.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;*)

(*procedure TIWRegion.SetVAlign(const Value: TIWGridVAlign);
begin
  // FVAlign := Value;
  Invalidate;
end;*)

function TIWCustomRegion.get_ExtraTagParams: TIWStringList;
begin
  result := FIWControlImplementation.get_ExtraTagParams;
end;

procedure TIWCustomRegion.set_ExtraTagParams(const Value: TIWStringList);
begin
  FIWControlImplementation.set_ExtraTagParams(Value);
end;

function TIWCustomRegion.get_OnHTMLTag: TIWOnHTMLTag;
begin
  result := FIWControlImplementation.OnHTMLTag;
end;

procedure TIWCustomRegion.set_OnHTMLTag(AValue: TIWOnHTMLTag);
begin
  FIWControlImplementation.setOnHTMLTag(AValue);
end;

procedure TIWCustomRegion.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FBorderOptions);
  LayoutMgr := nil; // Cleanup layout manager
  inherited;
  if ADispose then begin
    FreeAndNil(FIWControlImplementation);
    FreeAndNil(FIWBaseControlImplementation);
  end;
end;

function TIWCustomRegion.get_HTMLName: String;
begin
  result := FIWControlImplementation.HTMLName;
end;

function TIWCustomRegion.get_DoRefreshControl: Boolean;
begin
  result := FDoRefreshControl;
end;

procedure TIWCustomRegion.set_DoRefreshControl(AValue: Boolean);
begin
  FDoRefreshControl := AValue;
end;

function TIWCustomRegion.get_Clip: Boolean;
begin
  result := FIWControlImplementation.GetClip;
end;

function TIWCustomRegion.get_RenderSize: Boolean;
begin
  result := true;
end;

function TIWCustomRegion.get_ScriptEvents: TIWScriptEvents;
begin
  result := FIWControlImplementation.GetScriptEvents;
end;

function TIWCustomRegion.get_ZIndex: Integer;
begin
  result := FIWControlImplementation.GetZIndex;
end;

procedure TIWCustomRegion.set_Clip(AValue: Boolean);
begin
  FIWControlImplementation.SetClip(AValue);
end;

procedure TIWCustomRegion.set_RenderSize(AValue: Boolean);
begin
  // Constant
end;

procedure TIWCustomRegion.set_ScriptEvents(Value: TIWScriptEvents);
begin
  FIWControlImplementation.SetScriptEvents(Value);
end;

procedure TIWCustomRegion.set_ZIndex(AValue: Integer);
begin
  FIWControlImplementation.SetZIndex(AValue);
end;

function TIWCustomRegion.HintEvents(ATag: TIWHTMLTag{;
  const AHint: string}): string;
begin
  if ShowHint and (Hint <> '') then begin
    ATag.AddStringParam('title', StringReplace(Hint, '''', '&#39', [rfReplaceAll]));
  end;
end;

procedure TIWCustomRegion.HookEvents(APageContext: TIWPageContext40;
  AScriptEvents: TIWScriptEvents);
begin

end;

procedure TIWCustomRegion.set_Visible(Value: Boolean);
begin
  inherited;
  FIWControlImplementation.SetVisible(Value);
  FDoRefreshControl := true;
end;

procedure TIWCustomRegion.SetActiveControl(AControl: IIWHTML40Control);
begin
  IWHTML40Interfaces.SetActiveControl(Parent, AControl);
end;

procedure TIWCustomRegion.set_LayoutMgr(Value: TIWContainerLayout);
begin
  if Assigned(FLayoutMgr) then begin
    FLayoutMgr.SetContainer(nil);
    {$IFNDEF VSNET}
    FLayoutMgr.RemoveFreeNotification(Self);
    {$ENDIF}
  end;
  FLayoutMgr := Value;
  if Assigned(FLayoutMgr) then begin
    {$IFNDEF VSNET}
    FLayoutMgr.FreeNotification(Self);
    {$ENDIF}
    FLayoutMgr.SetContainer(Self);
  end;
end;

function TIWCustomRegion.get_LayoutMgr: TIWContainerLayout;
begin
  result := TIWContainerLayout(FLayoutMgr);
end;

procedure TIWCustomRegion.RenderComponents(AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext);
Var
  AStream: TStringStream;
  S: String;
begin
  ContainerContext := AContainerContext;
  inherited RenderComponents(ContainerContext, APageContext);
  TIWContainerLayout(ContainerContext.LayoutManager).ProcessControls(ContainerContext, TIWBaseHTMLPageContext(APageContext));
  AStream := TStringStream.Create('');
  with TIWContainerLayout(ContainerContext.LayoutManager) do try
    Process(AStream, ContainerContext, APageContext);
    SetContainer(nil);
  finally
    {$IFDEF CLR}
    // Workaroudn for Delphi 8 bug with DataString
    S := AnsiEncoding.GetString(AStream.Memory, Low(AStream.Memory), AStream.Size);
    {$ELSE}
    S := AStream.DataString;
    {$ENDIF}
    FRegionDiv.Contents.AddText(S);
    AStream.Free;
  end;
end;

function TIWCustomRegion.get_UseFrame: Boolean;
begin
  result := FUseFrame;
end;

procedure TIWCustomRegion.set_UseFrame(AValue: Boolean);
begin
  FUseFrame := AValue;
end;

procedure TIWCustomRegion.OnBorderChange(ASender: TObject);
begin
  {BorderOptions.Style := TIWContainerBorderOptions(ASender).Style;
  BorderOptions.NumericWidth := TIWContainerBorderOptions(ASender).NumericWidth;
  BorderOptions.Color := TIWContainerBorderOptions(ASender).Color;
  BorderOptions.Width := TIWContainerBorderOptions(ASender).Width;}
  RequestAlign;
  Invalidate;
end;

function TIWCustomRegion.GenerateControlPositions: String;
Var
  i: Integer;
begin
  result := '';
  with HTML40ContainerInterface(Self) do begin
    for i := 0 to IWComponentsCount - 1 do begin
      if SupportsInterface(Component[i], IIWHTML40Control) then begin
        with HTML40ControlInterface(Component[i]) do begin
          if DoRefreshControl then
            result := result + EOL 
              + 'top.' + HTMLName + Format('IWCL.SetRect(new top.Rect(%d, %d, %d, %d));',
                [HTMLLeft, HTMLTop, HTMLWidth, HTMLHeight]) + EOL;
        end;
      end;
      if SupportsInterface(Component[i], IIWHTML40Container) then begin
        result := result + EOL
          + HTML40ContainerInterface(Component[i]).GenerateControlPositions;
      end;
    end;
  end;
end;

{$IFNDEF VSNET}
procedure TIWCustomRegion.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  DoRefreshControl := (ALeft <> Left) or (ATop <> Top) or (AWidth <> Width) or (AHeight <> Height) or DoRefreshControl;
  inherited;
end;
{$ENDIF}

{$IFDEF VSNET}
procedure TIWCustomRegion.SetName(const AValue: String);
{$ELSE}
procedure TIWCustomRegion.SetName(const AValue: TComponentName);
{$ENDIF}
begin
  if Name <> 'IWFrameRegion' then
    inherited
  else
    Align := alClient;
end;

function TIWCustomRegion.ComponentContextClass: TIWBaseComponentContextClass;
begin
  result := TIWComponent40Context;
end;

// IIWContainer methods

function TIWCustomRegion.InitContainerContext(AWebApplication: TIWApplication): TIWContainerContext;
Var
  LLayoutMgr: TIWBaseContainerLayout;
begin
  result := IWBaseRenderContext.TIWContainerContext.Create(AWebApplication);
  LLayoutMgr := LayoutMgr;

  if LLayoutMgr = nil then begin
    LLayoutMgr := TIWLayoutMgrForm.Create(Self);
  end else begin
    LLayoutMgr.SetContainer(Self);
    if not LLayoutMgr.Able then begin
      LLayoutMgr := TIWLayoutMgrForm.Create(Self);
    end;
  end;

  LLayoutMgr.SetContainer(Self);
  result.LayoutManager := LLayoutMgr;
end;

procedure TIWCustomRegion.DoCreate;
begin
  if Assigned(OnCreate) and not IsDesignMode then begin
    OnCreate(Self);
  end;
end;

procedure TIWCustomRegion.DoHTMLTag(ATag: TIWHTMLTag);
begin
  FIWControlImplementation.DoHTMLTag(ATag);
end;

function TIWCustomRegion.get_Canvas: TCanvas;
begin
  result := FIWBaseControlImplementation.GetCanvas;
end;

function TIWCustomRegion.get_WebCursor: TIWCursor;
begin
  result := FIWControlImplementation.GetCursor;
end;

function TIWCustomRegion.get_WebFont: TIWFont;
begin
  result := FIWControlImplementation.Font;
end;

function TIWCustomRegion.get_HTMLHeight: Integer;
begin
  if IsDesignMode then begin
    result := FIWControlImplementation.HTMLHeight;
  end else begin
    result := FIWControlImplementation.HTMLHeight;
   if Assigned(FWebApplication) then
   begin { Hidden region have FWebApplication = nil but are resized at list if they are inside another region }
     Result := Result - iif(FWebApplication.Browser in [brNetscape6, brNetscape7], FBorderOptions.PixelWidth * 2, 0);
   end;
  end;
end;

function TIWCustomRegion.get_HTMLLeft: Integer;
begin
  result := FIWControlImplementation.GetHTMLLeft;
  {$IFDEF VSNET}
  {$ELSE}
  if Parent is TFrame then begin
    Result := Parent.Left;
  end;
  {$ENDIF}
end;

function TIWCustomRegion.get_HTMLTop: Integer;
begin
  result := FIWControlImplementation.GetHTMLTop;
  {$IFDEF VSNET}
  {$ELSE}
  if Parent is TFrame then begin
    Result := Parent.Top;
  end;
  {$ENDIF}
end;

function TIWCustomRegion.get_HTMLWidth: Integer;
begin
  if IsDesignMode then begin
    result := FIWControlImplementation.HTMLWidth;
  end else begin
    result := FIWControlImplementation.HTMLWidth;
    if Assigned(FWebApplication) then begin { Hidden region have FWebApplication = nil
       but are resized at list if they are inside another region }
       result := result -
         iif(FWebApplication.Browser in [brNetscape6, brNetscape7], FBorderOptions.PixelWidth * 2, 0);
    end;
  end;
end;

{$IFNDEF CLR}
function TIWCustomRegion.get_WebColor: TIWColor;
begin
  result := FIWControlImplementation.GetIWColor;
end;

procedure TIWCustomRegion.set_WebColor(Value: TIWColor);
begin
//  BorderOptions.BGColor := Value;
  FIWControlImplementation.SetIWColor(Value);
end;
{$ENDIF}

function TIWCustomRegion.getTabOrder: TIWTabOrder;
begin
  result := FIWControlImplementation.getTabOrder;
end;

procedure TIWCustomRegion.setTabOrder(AValue: TIWTabOrder);
begin
  FIWControlImplementation.SetTabOrder(AValue);
end;

{$IFNDEF VSNET}
procedure TIWCustomRegion.Invalidate;
begin
  inherited;
  FDoRefreshControl := true;
end;
{$ENDIF}

function TIWCustomRegion.MakeHTMLTag(ATag: TIWHTMLTag): string;
begin
  result := FIWControlImplementation.MakeHTMLTag(ATag);
end;

procedure TIWCustomRegion.PaintTo(ACanvas: TCanvas);
{$IFDEF VSNET}
Var
  i: Integer;
  Bmp: TBitmap;
  LX, LY: Integer;
{$ENDIF}
begin
  FIWBaseControlImplementation.PaintTo(ACanvas);
  {$IFDEF VSNET}
  if not Assigned(Site) and IsDesignMode then begin
    for i := 0 to IWComponentsCount - 1 do begin
      if SupportsInterface(Component[i], IIWBaseControl) then begin
        Bmp := TBitmap.Create; try
          LX := 0;
          LY := 0;
          if Component[i] is TIWNETBaseControl then begin
            with TIWNETBaseControl(Component[i]) do begin
              Bmp.Width := Width;
              Bmp.Height := Height;

              LX := Left;
              LY := Top;
            end;
          end;
          BaseControlInterface(Component[i]).PaintTo(Bmp.Canvas);

          ACanvas.Draw(LX, LY, Bmp);
        finally
          Bmp.Free;
        end;
      end;
    end;
  end;
  {$ENDIF}
end;

function TIWCustomRegion.ParentContainer: IIWBaseContainer;
begin
  result := FIWBaseControlImplementation.ParentContainer;
end;

function TIWCustomRegion.RenderCSSClass(
  AComponentContext: TIWBaseHTMLComponentContext): string;
begin
  result := FIWControlImplementation.RenderCSSClass(AComponentContext);
end;

{$IFDEF VSNET}
procedure TIWCustomRegion.AdjustClientRect(var Rect: Rectangle);
begin
  inherited AdjustClientRect(Rect);

  if IsDesignMode then begin
    Rect.Inflate(-FBorderOptions.PixelWidth, -FBorderOptions.PixelWidth);
  end;
end;

function TIWCustomRegion.GetClientRect: Rectangle;
begin
  result := inherited GetClientRect;

  if not IsDesignMode then begin
    result.Inflate(-FBorderOptions.PixelWidth, -FBorderOptions.PixelWidth);
  end;
end;

{$ELSE}
function TIWCustomRegion.GetClientRect: TRect;
begin
  result := inherited GetClientRect;

  if not IsDesignMode then begin
    InflateRect(result, -FBorderOptions.PixelWidth, -FBorderOptions.PixelWidth);
  end;
end;

procedure TIWCustomRegion.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  if IsDesignMode then begin
    InflateRect(Rect, -FBorderOptions.PixelWidth, -FBorderOptions.PixelWidth);
  end;
end;
{$ENDIF}

procedure TIWCustomRegion.RenderScripts(
  AComponentContext: TIWBaseHTMLComponentContext);
begin
  FIWControlImplementation.RenderScripts(AComponentContext);
  if AComponentContext.ContainerContext.LayoutManager is TIWLayoutMgrForm then begin
    with TIWPageContext40(AComponentContext.PageContext) do begin
      // if (WebApplication.Browser <> brOpera) then begin
        AddToIWCLInitProc('if (' + HTMLName + 'IWCL) {');
          AddToIWCLInitProc(HTMLName + 'IWCL.BorderWidth = ' + IntToStr(FBorderOptions.PixelWidth) + ';');
        AddToIWCLInitProc('}');
      // end;
    end;
  end;
end;

function TIWCustomRegion.RenderStyle(
  AComponentContext: TIWBaseHTMLComponentContext): string;
begin
  result := FIWControlImplementation.RenderStyle(AComponentContext);
end;

procedure TIWCustomRegion.set_WebCursor(AValue: TIWCursor);
begin
  FIWControlImplementation.SetCursor(AValue);
end;

procedure TIWCustomRegion.set_WebFont(AValue: TIWFont);
begin
  FIWControlImplementation.Font := AValue;
end;

function TIWCustomRegion.SupportsInput: Boolean;
begin
  result := FIWControlImplementation.SupportsInput;
end;

function TIWCustomRegion.SupportsSubmit: Boolean;
begin
  result := FIWControlImplementation.SupportsSubmit;
end;

function TIWCustomRegion.WantsSubmitValidation: Boolean;
begin
  result := FIWControlImplementation.WantsSubmitValidation;
end;

function TIWCustomRegion.get_Visible: Boolean;
begin
  result := FIWControlImplementation.GetVisible;
end;

function TIWCustomRegion.InitPaintHandler: TIWPaintHandler;
begin
  result := TIWBaseControl.CreatePaintHandler(ClassName, Self);
end;

function TIWCustomRegion.get_ComponentName: String;
begin
  result := HTMLName;
end;

procedure TIWCustomRegion.AddFreeNotifier(AObject: TObject);
begin
end;

procedure TIWCustomRegion.RemoveFreeNotifier(AObject: TObject);
begin
end;

function TIWCustomRegion.get_SupportsPartial: Boolean;
begin
  result := FIWControlImplementation.get_SupportsPartial;
end;

procedure TIWCustomRegion.set_SupportsPartial(AValue: BOolean);
begin
  FIWControlImplementation.set_SupportsPartial(AValue);
end;

{$IFDEF VSNET}
{ TIWUserControl }

procedure TIWUserControl.InitControl;
begin
  inherited;
  FSetSizeCount := 0;
  // Inc(GIsLoading);
end;

function TIWUserControl.ContainerPrefix: string;
begin
  result := HTMLName;
end;

function TIWUserControl.InitPaintHandler: TIWPaintHandler;
begin
  // Use IWRegion paint handler
  result := TIWBaseControl.CreatePaintHandler('TIWRegion', Self);
end;

procedure TIWUserControl.Loaded;
begin
  inherited Loaded;
  
  if Assigned(FOnCreate) then begin
    FOnCreate(Self);
  end;
end;

procedure TIWUserControl.setSize(Value: Size);
begin
  inherited;
  if not IsCreating then begin
    Inc(FSetSizeCount);
    // if FSetSizeCount = 1 then Dec(gIsLoading);
    if not RootLoading and (FSetSizeCount = 2) then begin
      // Dec(gIsLoading);
      ForceAlign;
    end;
  end;
end;

{procedure TIWUserControl.RequestAlign;
begin
  if DesignMode then begin
    inherited;
  end;
end;}

procedure TIWUserControl.Paint;
Var
  i: Integer;
  Bmp: TBitmap;
  LX, LY: Integer;
begin
  inherited;
  for i := 0 to IWComponentsCount - 1 do begin
    if SupportsInterface(Component[i], IIWBaseControl) then begin
      Bmp := TBitmap.Create; try
        LX := 0;
        LY := 0;
        if Component[i] is TIWNETBaseControl then begin
          with TIWNETBaseControl(Component[i]) do begin
            Bmp.Width := Width;
            Bmp.Height := Height;

            LX := Left;
            LY := Top;
          end;
        end;
        BaseControlInterface(Component[i]).PaintTo(Bmp.Canvas);

        Canvas.Draw(LX, LY, Bmp);
      finally
        Bmp.Free;
      end;
    end;
  end;
end;
{$ENDIF}

{ TIWRegion }

function TIWRegion.ContainerPrefix: string;
begin
  {$IFNDEF VSNET}
  if Owner is TFrame then begin
    result := UpperCase(TFrame(Owner).Name);
  end else
  {$ENDIF}
    if SupportsInterface(Parent, IIWBaseContainer) then begin
      result := BaseContainerInterface(Parent).ContainerPrefix;
    end else begin
      result := UpperCase(Name);
    end;
end;

end.
