unit IWTemplateProcessorHTML32;

interface

Uses
  {$IFDEF VSNET}
  System.ComponentModel,
  System.Drawing,
  IWTemplateFiles32Converter,
  {$ENDIF}
  Classes,
  IWContainer32Layout, IWRenderContext, IWTypes, IWBaseRenderContext;

Type
  {$IFDEF VSNET}
  [TypeConverter(typeof(TIWTemplateFiles32Converter))]
  {$ENDIF}
  TIWTemplateFiles32 = class(TPersistent)
  protected
    FDefault: string;
    //
    procedure AssignTo(ADest: TPersistent); override;
  public
    constructor Create(ADefault: String); overload;
  published
    property Default: string read FDefault write FDefault;
  end;

  {$IFDEF VSNET}
  {$R icons\Atozed.Intraweb.IWTemplateProcessorHTML32.TIWTemplateProcessorHTML32.bmp}
  TIWTemplateProcessorHTML32 = class;
  [ToolboxItem(true), ToolboxBitmap(typeof(TIWTemplateProcessorHTML32), 'TIWTemplateProcessorHTML32.bmp')]
  {$ENDIF}
  TIWTemplateProcessorHTML32 = class(TIWContainer32Layout)
  protected
    FMasterFormTag: Boolean;
    FOnAfterProcess: TIWTemplateProcessEvent;
    FOnBeforeProcess: TIWTemplateProcessEvent;
    FOnUnknownTag: TIWUnknownTagEvent;
    FTagType: TIWTemplateLayoutHTMLTagType;
    FTemplates: TIWTemplateFiles32;
    //
    function DoUnknownTag(const AName: string): string;
    procedure SetTemplates(const AValue: TIWTemplateFiles32);

    // function GetComponents: TList; override;

    // procedure Loaded; override;

    procedure InitControl; override;
  {$IFDEF CLR}
  strict protected
  {$ELSE}
  protected
  {$ENDIF}
    procedure Dispose(ADispose: Boolean); override;
  public
    function Able: Boolean; override;
    procedure Process(AStream: TStream; AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext); override;
    procedure ProcessStream(AStream: TStream; ATemplateStream: TStream; AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext);
    function TemplatePathname: string;
  published
    property MasterFormTag: Boolean read FMasterFormTag write FMasterFormTag;
    property TagType: TIWTemplateLayoutHTMLTagType read FTagType write FTagType;
    property Templates: TIWTemplateFiles32 read FTemplates write SetTemplates;
    property OnUnknownTag: TIWUnknownTagEvent read FOnUnknownTag write FOnUnknownTag;
    property OnBeforeProcess: TIWTemplateProcessEvent read FOnBeforeProcess write FOnBeforeProcess;
    property OnAfterProcess: TIWTemplateProcessEvent read FOnAfterProcess write FOnAfterProcess;
  end;

implementation

Uses
  CopyPrsr, SysUtils,
  {$IFDEF VCL6PRABOVE}
  Variants,
  {$ENDIF}
  IWUtils, IWGlobal, IWHTMLTag, IWHTML32Interfaces, IWApplication, IWBaseInterfaces,
  IWBaseHTMLInterfaces, SWStrings, SWSystem, IWBaseContainerLayout,
  IWMarkupLanguageTag;

{ TIWTemplateProcessorHTML32 }

function TIWTemplateProcessorHTML32.Able: Boolean;
begin
  Result := False;
  if Enabled then begin
    Result := FileExists(TemplatePathname) or Assigned(FOnBeforeProcess);
  end;
end;

procedure TIWTemplateProcessorHTML32.InitControl;
begin
  inherited;
  FMasterFormTag := True;
  FTemplates := TIWTemplateFiles32.Create;
end;

procedure TIWTemplateProcessorHTML32.Dispose(ADispose: Boolean);
begin
  FreeAndNil(FTemplates);
  inherited;
end;

function TIWTemplateProcessorHTML32.DoUnknownTag(const AName: string): string;
begin
  Result := '';
  if Assigned(OnUnknownTag) then begin
    OnUnknownTag(AName, Result);
  end;
end;

(*procedure TIWTemplateProcessorHTML32.Loaded;
var
  s: string;
begin
  inherited Loaded;
  if not IsDesignMode then begin
    if Length(Templates.Default) = 0 then begin
      s := Copy(Container.ContainerName, 2, MaxInt) + '.html';
      if Assigned(GServerController) and FileExists(GServerController.TemplateDir + s) then begin
        FTemplates.Default := s;
      end;
    end;
  end;
end;*)

procedure TIWTemplateProcessorHTML32.Process(AStream: TStream;
  AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext);
var
  LSrcStream: TStream;
begin
  if FileExists(TemplatePathname) then begin
    LSrcStream := TFileStream.Create(TemplatePathname, fmOpenRead + fmShareDenyWrite);
    try
      if Assigned(FOnBeforeProcess) then begin
        OnBeforeProcess(LSrcStream);
      end;

      ProcessStream(AStream, LSrcStream, AContainerContext, APageContext);
    finally
      FreeAndNil(LSrcStream);
    end;
  end else begin
    LSrcStream := nil;
    if Assigned(FOnBeforeProcess) then begin
      OnBeforeProcess(LSrcStream);
    end;
    if Assigned(LSrcStream) then begin
      try
        ProcessStream(AStream, LSrcStream, AContainerContext, APageContext);
      finally
        FreeAndNil(LSrcStream);
      end;
    end;
  end;
  if Assigned(FOnAfterProcess) then begin
    OnAfterProcess(AStream);
  end;
end;

procedure TIWTemplateProcessorHTML32.ProcessStream(AStream, ATemplateStream: TStream;
 AContainerContext: TIWContainerContext; APageContext: TIWBasePageContext);
var
  i: Integer;
  LControl: IIWHTML32Control;
  LComponent: IIWBaseHTMLComponent;
  LHeaderHasBeenWritten: Boolean;
  LParser: TCopyParser;
  LTagName: string;
  LTagParams: string;
  LName: string;
  LValue: String;
  LTerminated: Boolean;
  LWebApp: TIWApplication;
//  LHTMLStream: TStringStream;
//  LChar: Char;
  LContext: TIWPageContext32;

  function ProcessTag(AParser: TCopyParser; const AEndTagChar1: Char;
    const AEndTagChar2: Char): string;
  var
    LCtrlName: string;
    LHTML: TIWHTMLTag;
    LComp: TComponent;
  begin
    Result := '';
    with AParser do begin
      SkipToken(True);
      LCtrlName := TokenString;
      if Length(LCtrlName) > 0 then begin
        if AContainerContext.IsValidComponent(LCtrlName) then begin
          lComp := AContainerContext.Components[LCtrlName];
          LHTML := TIWBaseHTMLComponentContext(AContainerContext.ComponentContext[LCtrlName]).HTMLTag;
          LControl := HTML32ControlInterface(lComp);
          LComponent := BaseHTMLComponentInterface(lComp);
          if Assigned(LHTML) and BaseControlInterface(lComp).Visible then begin
            result := ''; try
              Result := LComponent.MakeHTMLTag(LHTML);
              if not MasterFormTag then begin
                if LComponent.SupportsInput then begin
                  with LContext.FormTag.Contents.AddTag('INPUT') do begin
                    AddStringParam('TYPE', 'HIDDEN');
                    AddStringParam('NAME', LComponent.HTMLName);
                    AddStringParam('ID', LComponent.HTMLName);
                  end;
                end;
              end;
            except on EAbort do end;
          end;
        end else begin
          Result := DoUnknownTag(LCtrlName);
        end;
      end;
      SkipToken(True);
      if TagType <> ttBorland then begin
        CheckToken(AEndTagChar1);
        SkipToken(True);
      end;
      CheckToken(AEndTagChar2);
      SkipToken(True);
    end;
  end;

  procedure SkipAndOutput(AParser: TCopyParser);
  begin
    with AParser do begin
      CopyTokenToOutput;
      SkipToken(True);
    end;
  end;

  procedure SkipOverNextHTMLTag(AParser: TCopyParser);
  begin
    with AParser do begin
      SkipToToken('<');
      SkipToken(True);
      CheckToken('/');
      SkipToToken('>');
      SkipToken(True);
    end;
  end;

  procedure WriteHead;
  begin
    LHeaderHasBeenWritten := True;
    StreamWrite(AStream, '<HEAD>' + EOL 
      + iif( GServerController.CharSet , '<META HTTP-EQUIV="Content-Type" CONTENT="text/html; CHARSET='+GServerController.CharSet+'">' + EOL)
      + HeadContent);
  end;

begin
  LContext := TIWPageContext32(APageContext);
//  LHTMLStream := TStringStream.Create('');
{  while ATemplateStream.Read(LChar, 1) > 0 do begin
    case LChar of
      '''': LHTMLStream.WriteString( '&#39;' );
    else;
      LHTMLStream.WriteString( LChar );
    // end;
  end;}
//  LHTMLStream.Position := 0;

  LParser := TCopyParser.Create(ATemplateStream, AStream); try
    LHeaderHasBeenWritten := false;

    LTerminated := False;
    if Assigned (GGetWebApplicationThreadVar) then begin
      LWebApp := GGetWebApplicationThreadVar;
      LTerminated := Assigned(LWebApp) and LWebApp.Terminated;
    end;

    with LParser do begin
      while Token <> toEOF do begin
        case Token of
          '<': begin
            // SkipToken first in case of short circuit
            if (SkipToken(True) = '#') and (TagType = ttBorland) then begin
              StreamWrite(AStream, ProcessTag(LParser, '#', '>'));
            end else begin
              // <!DOCTYPE - etc. this is why TokenString + SkipTo
              LTagName := TokenString;
              // Must be separate from above, SkipToToken modfies result value of TokenString
              LTagName := LTagName + SkipToToken('>');
              //
              i := Pos(' ', LTagName);
              if i > 0 then begin
                LTagParams := Copy(LTagName, i + 1, MaxInt);
                SetLength(LTagName, i - 1);
                LTagParams := StringReplace(LTagParams, '../files/', LContext.URLBase + '/files/'
                 , [rfReplaceAll, rfIgnoreCase]);
              end else begin
                LTagParams := '';
              end;
              if AnsiSameText(LTagName, 'BODY') then begin
                while Length(LTagParams) > 0 do begin
                  LName := Trim(Fetch(LTagParams));
                  LValue := LName;
                  LName := Trim(Fetch(LValue, '='));
                  if AnsiPos('"', LValue) > 0 then begin
                    Fetch(LValue, '"');
                    if AnsiPos('"', LValue) = 0 then begin
                      LTagParams := LValue + LTagParams;
                      LValue := Fetch(LTagParams, '"');
                    end else begin
                      LValue := Fetch(LValue, '"');
                    end;
                  end else begin
                    if AnsiPos('''', LValue) > 0 then begin
                      Fetch(LValue, '''');
                      if AnsiPos('''', LValue) = 0 then begin
                        LTagParams := LValue + LTagParams;
                        LValue := Fetch(LTagParams, '''');
                      end else begin
                        LValue := Fetch(LValue, '''');
                      end;
                    end;
                  end;
                  if LName <> '' then begin
                    if AnsiSameText(LName, 'BGCOLOR') then begin
                      if LContext.BodyTag.Params.Values['BGCOLOR'] = '' then begin
                        LContext.BodyTag.AddStringParam('BGCOLOR', LValue);
                      end;
                    end else begin
                      if LValue <> '' then begin
                        LContext.BodyTag.AddStringParam(LName, LValue);
                      end else begin
                        LContext.BodyTag.Add(LName);
                      end;
                    end;
                  end;
                end;
                if not LHeaderHasBeenWritten then begin
                  WriteHead;
                end;
                LContext.BodyTag.ClosingTag := cbFalse;
                StreamWrite(AStream, LContext.BodyTag.Render);
                SkipToken(True);
                if MasterFormTag and not LTerminated then begin
                  LContext.FormTag.ClosingTag := cbFalse;
                  StreamWrite(AStream, LContext.FormTag.Render);
                end;
              end else if AnsiSameText(LTagName, '/BODY') then begin
                if not LTerminated then begin
                  if not MasterFormTag then begin
                    LContext.FormTag.ClosingTag := cbFalse;
                    StreamWrite(AStream, LContext.FormTag.Render);
                    // SkipToken(True);// Added. Fixes CR bug. 26/04
                  end;
                  // StreamWrite(AStream, FormContent);
                  StreamWrite(AStream, '</FORM>' + EOL);
                end;
                StreamWrite(AStream, '</BODY>');
                SkipToken(True); // Added. Fixes CR bug. 26/04
              end else if AnsiSameText(LTagName, 'HEAD') then begin
                if not LHeaderHasBeenWritten then begin
                  WriteHead;
                end;
                SkipToken(True);
              end else if AnsiSameText(LTagName, 'TITLE') then begin
                if Length(LContext.Title) > 0 then begin
                  SkipOverNextHTMLTag(LParser);
                  StreamWriteLn(AStream, '<TITLE>' + LContext.Title + '</TITLE>');
                end else begin
                  StreamWrite(AStream, '<TITLE>');
                  SkipToken(True); // Added. Fixes CR bug. 26/04
                end;
              end
              else begin
                StreamWrite(AStream, '<' + Trim(LTagName + ' ' + LTagParams) + '>');
                SkipToken(True);// Added. Fixes CR bug. 26/04
              end;
            end;
          end;
          '{': begin
            LValue := SkipTo(2);
            if (LValue = '{%') and (TagType = ttIntraWeb) then begin
              // SkipToken first in case of short circuit
              // if (SkipToken(True) = '%') and (TagType = ttIntraWeb) then begin
              //SkipToken(True);
              StreamWrite(AStream, ProcessTag(LParser, '%', '}'));
            end else begin
              // StreamWrite(AStream, LValue);
              SkipAndOutput(LParser);
            end;
          end else begin
            SkipAndOutput(LParser);
          end;
        end;
      end;
    end;
  finally
    FreeAndNil(LParser);
//    FreeAndNil(LHTMLStream);
  end;
end;

procedure TIWTemplateProcessorHTML32.SetTemplates(const AValue: TIWTemplateFiles32);
begin
  FTemplates.Assign(AValue);
end;

function TIWTemplateProcessorHTML32.TemplatePathname: string;
Var
  S: String;
begin
  if Templates.Default = '' then begin
    // This is for subtemplates of regions. Right now containername returns:
    // formName.regionName. We rename it to formNameRegionName and that way
    // if no name is specified for the templates for regions, it uses this as
    // the default one
    if Assigned(Container) then begin
      s := StringReplace(Container.ContainerName, '.', '', []);
      s := s + '.html';
    end else begin
      s := '';
    end;
    if Assigned(GServerController) and FileExists(GServerController.TemplateDir + s) then begin
      FTemplates.Default := s;
    end;
  end;
  Result := GServerController.TemplateDir + FTemplates.Default;
end;

{ TIWTemplateFiles32 }

procedure TIWTemplateFiles32.AssignTo(ADest: TPersistent);
begin
  if ADest is TIWTemplateFiles32 then begin
    with TIWTemplateFiles32(ADest) do begin
      Default := Self.Default;
    end;
  end else begin
    inherited;
  end;
end;

constructor TIWTemplateFiles32.Create(ADefault: String);
begin
  inherited Create;

  FDefault := ADefault;
end;

end.
